/*
 *************************************************************************************************************
 * PickSortEditierspalte.cpp     
 *
 * Die Struktur einer editierbaren Spalte, die Bilderelemente aufnehmen kann.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortEditierspalte.h"


short EditierspalteEventBoxClass::EditierspalteAnzahl = 0;
std::vector<Glib::ustring> EditierspalteEventBoxClass::EditierspaltenNamen;

EditierspalteEventBoxClass* EditierspalteEventBoxClass::EditierspalteAktuell = NULL;


EditierspalteEventBoxClass::EditierspalteEventBoxClass()
{
 // Element mit dem X-Window verbinden:
 set_events(Gdk::BUTTON_PRESS_MASK);
 // Callback-Funktion:
 signal_button_press_event().connect
  (sigc::mem_fun(*this, &EditierspalteEventBoxClass::EditierspalteButtonPressEventHandler), false); 
 // Anzahl der Spalten mitzaehlen:
 ++EditierspalteAnzahl;
 // Und die eigene Nummer der vorliegenden Spalte zuweisen:
 EditierspalteNummerHier = EditierspalteAnzahl - 1;
 // Name der Spalte = Verzeichnisname einrichten - hier die anfaengliche Spaltenueberschrift:
 EditierspaltenNamen.push_back(_("void"));
 // Startwerte der Zustands-Kontrollvariablen setzen:
 GenerellesEditierspalteHingeschoben.push_back(false);
 GenerellesEditierspalteUmsortiert.push_back(false);
 // Grafikelement-Ueberschrift einrichten: 
 EditierspalteLabel.set_text(EditierspaltenNamen[EditierspalteNummerHier]); 
 EditierspalteLabel.set_ellipsize(Pango::ELLIPSIZE_START);
 EditierspalteLabel.set_hexpand(true);
 EditierspalteLabel.set_vexpand(false);
 EditierspalteLabel.override_color(GenerellesSpalteSchrift);
 // Start- und Endeelement anlegen. Das Endelement wird sichtbar, das Startelement jedoch nicht.
 // Alle Elemente sind nicht mit Gtk::manage() angelegt. Dies deshalb, weil z.B. ueber die Neuladenfunktion
 // oder am Ende des Programmes die Deallokation gezielt ausgefuehrt werden soll.
 // Erstes Element:
 Start = new ElementEventBoxClass(" ", " ", false, EditierspalteNummerHier);
 Start->ElementDavor = NULL;
 // Letztes Element:
 Ende = new ElementEventBoxClass("~", "~", true, EditierspalteNummerHier); 
 Ende->ElementDanach = NULL;
 // Elemente verketten:
 Start->ElementDanach = Ende;
 Ende->ElementDavor = Start;
 // Abschlusselement in Grid einsetzen.
 // Grid zuerst dynamisch allokieren, so dass man das Grid gezielt loeschen kann:
 EditierspalteGrid = Gtk::manage(new Gtk::Grid()); 
 EditierspalteGrid->attach(*Ende, 1, 1, 1, 1);
 EditierspalteRahmen.add(*EditierspalteGrid);
 EditierspalteRahmen.set_vexpand(true);
 EditierspalteRahmen.set_shadow_type(Gtk::SHADOW_NONE);
 // Das Grid mit dem Rahmen kann laenger werden als der Bildschirm hoch ist:
 EditierspalteScrolledWindow.add(EditierspalteRahmen);
 EditierspalteScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
 // Elemente in den aeusseren Rahmen einstellen:
 EditierspalteRahmenGrid.attach(EditierspalteLabel, 1, 1, 1, 1);
 EditierspalteRahmenGrid.attach(EditierspalteScrolledWindow, 1, 2, 1, 1);
 // Das ganze noch in die EventBox:
 add(EditierspalteRahmenGrid);
 // Anfaenglicher Auswahlzustand anzeigen: 
 if (!EditierspalteAktuell)
  {
   override_background_color(GenerellesSpalteFarbeGewaehlt); 
   EditierspalteAktuell = this;
  }
 else
  {
   override_background_color(GenerellesSpalteFarbeNormal);
  }
 show_all();
}  // EditierspalteEventBoxClass


EditierspalteEventBoxClass::~EditierspalteEventBoxClass()
{
 ElementEventBoxClass *Zeiger;
 
 while (Start != Ende) 
  {
   Zeiger = Start;
   Start = Start->ElementDanach;
   delete Zeiger;   
  }
 delete Ende;
}  // ~EditierspalteFrameClass


void EditierspalteEventBoxClass::EditierspalteNeu()
{
 // Sicherstellen, dass ein Benutzerklick nicht die aktuelle Spaltenauswahl veraendert, solange die Lade-
 // aktionen laufen. Sonst waere ja die Situation unsynchron geworden:
 EditierspalteEventBoxClass*  EditierspalteAktuellSpeicher = EditierspalteAktuell;
 // Logikvariable fuer die Steuerung des Programmablaufes:
 bool NeuMachen = false;
 // Bei der Reihenfolge der Abfragen die Prios der Verriegelungen beachten:
 // Wenn die Spalte leer ist, dann geht ja alles:
 if (EditierspalteAktuellSpeicher->GetSpalteIstLeer())
  {
   NeuMachen = true;
  }
 else
  {
   // - Verriegelung Prio 1 - mindestens ein Element wurde in die Spalte geschoben:
   if (GenerellesEditierspalteHingeschoben[EditierspalteAktuellSpeicher->EditierspalteNummerHier])
    { 
     Gtk::MessageDialog *EingefuegtDialog =
      new Gtk::MessageDialog(_("Emptying impossible!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
     EingefuegtDialog->set_secondary_text
      (_("Pictures have been inserted into column.\nSave column first."));
     EingefuegtDialog->run();
     delete EingefuegtDialog;
    }
   else
    {
     // - Verriegelung Prio 2 - mindestens ein Element wurde umsortiert oder umbenannt resp. umbewertet:
     bool UmbewertetUmbenannt = false;
     for (ElementEventBoxClass* Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
          Zeiger != EditierspalteAktuellSpeicher->Ende;
          Zeiger = Zeiger->ElementDanach)
      {
       if (Zeiger->GetElementUmbenannt() || Zeiger->GetElementUmbewertet())
        {
         UmbewertetUmbenannt = true;
         break;
        }
      }
     if (GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier] ||
         UmbewertetUmbenannt)
      {
       Gtk::MessageDialog *LadenDialog = 
        new Gtk::MessageDialog(_("Emptying?"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
       LadenDialog->set_secondary_text
        (_("Column has been resorted, files have been renamed or rerated.\nCommand discards this premliminary work."));  
       if (LadenDialog->run() == Gtk::RESPONSE_YES)
        {
         delete LadenDialog;
         // Spalte zuruecksetzen:
         NeuMachen = true;  
        }
       else
        {
         delete LadenDialog;
        }
      }
     else
      {
       //Spalte zuruecksetzen:
       NeuMachen = true;
      }
    }
  }
 // Und jetzt auch wirklich machen:
 if (NeuMachen)
  {
   ElementEventBoxClass* Zeiger1;
   ElementEventBoxClass* Zeiger2;
   EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = _("void");
   EditierspalteAktuellSpeicher->EditierspalteLabel.set_text
    (EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier]);
   // Altes Grid loeschen:
   delete EditierspalteAktuellSpeicher->EditierspalteGrid;     
   // Dann die Elemente einzeln deallokieren, Start und Ende bleiben aber bestehen:
   Zeiger1 = EditierspalteAktuellSpeicher->Start->ElementDanach;
   while (Zeiger1 != EditierspalteAktuellSpeicher->Ende) 
    {
     Zeiger2 = Zeiger1;
     Zeiger1 = Zeiger1->ElementDanach;
     delete Zeiger2;
    }
   EditierspalteAktuellSpeicher->Start->ElementDanach = EditierspalteAktuellSpeicher->Ende;
   EditierspalteAktuellSpeicher->Ende->ElementDavor = EditierspalteAktuellSpeicher->Start;    
   // Neues Grid anlegen. Dennoch wieder manage() wegen des Programmendes:
   EditierspalteAktuellSpeicher->EditierspalteGrid = Gtk::manage(new Gtk::Grid());
   EditierspalteAktuellSpeicher->EditierspalteRahmen.add(*(EditierspalteAktuellSpeicher->EditierspalteGrid));      
   // Den Verzeichnisnamen auch ins Ende-Element schreiben:
   EditierspalteAktuellSpeicher->Ende->ElementUeberschreibeVerzeichnisNeu
    (EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier]);
   // Das Abschlusselement ins Grid laden: 
   EditierspalteAktuellSpeicher->EditierspalteGrid->attach
    (*(EditierspalteAktuellSpeicher->Ende), 1, 1, 1, 1);      
   EditierspalteAktuellSpeicher->show_all();
   // Spaltenzustandsvariablen entsprechend dem erfolgreichen Laden setzen:
   GenerellesEditierspalteHingeschoben[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;
   GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;   
  }
}  // EditierspalteNeu


void EditierspalteEventBoxClass::EditierspalteLaden()
{
 // Sicherstellen, dass ein Benutzerklick nicht die aktuelle Spaltenauswahl veraendert, solange die Lade-
 // aktionen laufen. Sonst waere ja die Situation unsynchron geworden:
 EditierspalteEventBoxClass*  EditierspalteAktuellSpeicher = EditierspalteAktuell; 
 // Bei der Reihenfolge der Abfragen die Prios der Verriegelungen beachten:
 // Zuerst: Wenn die Spalte leer ist, dann kann man sofort beladen:
 if (EditierspalteAktuellSpeicher->GetSpalteIstLeer())
  {
   EditierspalteFilesLaden(EditierspalteAktuellSpeicher);
  }
 else
  {
   // - Verriegelung Prio 1 - mindestens ein Element wurde in die Spalte geschoben:
   if (GenerellesEditierspalteHingeschoben[EditierspalteAktuellSpeicher->EditierspalteNummerHier])
    {
     Gtk::MessageDialog *EingefuegtDialog =
      new Gtk::MessageDialog(_("Reloading impossible!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
     EingefuegtDialog->set_secondary_text
      (_("Pictures have been inserted into column.\nSave column first."));
     EingefuegtDialog->run();
     delete EingefuegtDialog;
    }
   else
    {
     // - Verriegelung Prio 2 - mindestens ein Element wurde umsortiert oder umbenannt resp. umbewertet:
     bool UmbewertetUmbenannt = false;
     for (ElementEventBoxClass* Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
          Zeiger != EditierspalteAktuellSpeicher->Ende;
          Zeiger = Zeiger->ElementDanach)
      {
       if (Zeiger->GetElementUmbenannt() || Zeiger->GetElementUmbewertet())
        {
         UmbewertetUmbenannt = true;
         break;
        }
      }
     if (GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier] ||
         UmbewertetUmbenannt)
      {
       Gtk::MessageDialog *LadenDialog = 
        new Gtk::MessageDialog(_("Reload?"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
       LadenDialog->set_secondary_text
        (_("Column has been resorted, files have been renamed or rerated.\nCommand discards this premliminary work."));  
       if (LadenDialog->run() == Gtk::RESPONSE_YES)
        {
         delete LadenDialog;
         EditierspalteFilesLaden(EditierspalteAktuellSpeicher);    
        }
       else
        {
         delete LadenDialog;
        }
      }
     // - Verriegelung Prio 3 - Spalte sogleich neu beladen:
     else
      {
       EditierspalteFilesLaden(EditierspalteAktuellSpeicher);      
      }
    }
  }
}  // EditierspalteLaden


bool EditierspalteEventBoxClass::EditierspalteVerzeichnisDoppelt
 (EditierspalteEventBoxClass*  EditierspalteAktuellSpeicher, const Glib::ustring& NeuesVerzeichnis)
{    
 // Zwei for-Schleifen: Die bis zum aktuellen Element und die nach dem aktuellen Element:
 for (int Zaehler = 0; Zaehler < EditierspalteAktuellSpeicher->EditierspalteNummerHier; ++Zaehler)
  {
   if (NeuesVerzeichnis == EditierspaltenNamen[Zaehler])
    { 
     return true;
    }
  }
 for (int Zaehler = EditierspalteAktuellSpeicher->EditierspalteNummerHier + 1;
          Zaehler < EditierspalteAnzahl;
          ++Zaehler)
  {
   if (NeuesVerzeichnis == EditierspaltenNamen[Zaehler])
    {
     return true;
    }
  }
 return false; 
}  // EditierspalteVerzeichnisDoppelt


void EditierspalteEventBoxClass::EditierspalteFilesLaden
 (EditierspalteEventBoxClass*  EditierspalteAktuellSpeicher)
{
 ElementEventBoxClass* Zeiger1;
 ElementEventBoxClass* Zeiger2;
 FortschrittDialogClass* FortschrittBalkenZeiger;
 int Zaehler;

 // Einen temporaeren Anschluss zum Filesystem herstellen:
 FilehandlingClass FileSystem;
 // Bilderdaten auswaehlen lassen und laden:
 int FileAnzahl = FileSystem.FilehandlingFilesAuswaehlen();
 if (FileAnzahl != 0)
  {
   // Verhindern, dass ein Verzeichnis einer anderen Spalte nochmals geladen wird:
   if (EditierspalteVerzeichnisDoppelt 
        (EditierspalteAktuellSpeicher, Glib::filename_to_utf8(FileSystem.GetFilehandlingVerzeichnisName())))
    {
     // Benutzer darauf hinweisen und Abbruch:
     Gtk::MessageDialog *DoppeltDialog =
      new Gtk::MessageDialog(_("Reloading impossible!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
     DoppeltDialog->set_secondary_text
      (_("Directoy has been loaded into other column already.\nUnsynchonicity is imminent."));
     DoppeltDialog->run();
     delete DoppeltDialog;
     return;
    }
   // Files laden:   
   else
    { 
     EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = 
      Glib::filename_to_utf8(FileSystem.GetFilehandlingVerzeichnisName());
     EditierspalteAktuellSpeicher->EditierspalteLabel.set_text
      (EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier]);
     FortschrittBalkenZeiger = new FortschrittDialogClass("PickSort", _("Loading and extracting files..."));
     FortschrittBalkenZeiger->SetFortschrittAnzeige(0.0);
     // Altes Grid loeschen:
     delete EditierspalteAktuellSpeicher->EditierspalteGrid;     
     // Dann die Elemente einzeln deallokieren, Start und Ende bleiben aber bestehen:
     Zeiger1 = EditierspalteAktuellSpeicher->Start->ElementDanach;
     while (Zeiger1 != EditierspalteAktuellSpeicher->Ende) 
      {
       Zeiger2 = Zeiger1;
       Zeiger1 = Zeiger1->ElementDanach;
       delete Zeiger2;
      }
     EditierspalteAktuellSpeicher->Start->ElementDanach = EditierspalteAktuellSpeicher->Ende;
     EditierspalteAktuellSpeicher->Ende->ElementDavor = EditierspalteAktuellSpeicher->Start;     
     // Neues Grid anlegen. Dennoch wieder manage() wegen des Programmendes:
     EditierspalteAktuellSpeicher->EditierspalteGrid = Gtk::manage(new Gtk::Grid());     
     EditierspalteAktuellSpeicher->EditierspalteRahmen.add(*(EditierspalteAktuellSpeicher->EditierspalteGrid));  
     // Die Fileelemente laden:
     for (Zaehler = 1; Zaehler <= FileAnzahl; ++Zaehler)
      {
       // Ein Fileelement einladen. Der Datensatz hat bereits Verkettungselementformat:
       Zeiger1 = FileSystem.FilehandlingElementLaden(EditierspalteAktuellSpeicher->EditierspalteNummerHier);
       // Element in die Liste vor dem Ende einknuepfen: 
       Zeiger1->ElementDanach = EditierspalteAktuellSpeicher->Ende;
       Zeiger1->ElementDavor = EditierspalteAktuellSpeicher->Ende->ElementDavor;
       EditierspalteAktuellSpeicher->Ende->ElementDavor->ElementDanach = Zeiger1;
       EditierspalteAktuellSpeicher->Ende->ElementDavor = Zeiger1;
       // Element ins Grid laden:
       EditierspalteAktuellSpeicher->EditierspalteGrid->attach(*Zeiger1, 1, Zaehler, 1, 1);    
       // Fortschritt anzeigen:
       FortschrittBalkenZeiger->SetFortschrittAnzeige(((float)Zaehler-1.0) / FileAnzahl);  
       if (FortschrittBalkenZeiger->GetFortschrittEndeAnforderung()) break;
      }       
     // Den Verzeichnisnamen auch ins Ende-Element schreiben:
     EditierspalteAktuellSpeicher->Ende->ElementUeberschreibeVerzeichnisNeu
      (EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier]);
     // Das Abschlusselement ins Grid laden: 
     EditierspalteAktuellSpeicher->EditierspalteGrid->attach
      (*(EditierspalteAktuellSpeicher->Ende), 1, Zaehler, 1, 1);      
     EditierspalteAktuellSpeicher->show_all();
     // Spaltenzustandsvariablen entsprechend dem erfolgreichen Laden setzen:
     GenerellesEditierspalteHingeschoben[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;
     GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;
     delete FortschrittBalkenZeiger;     
    }
  }  
}  // EditierspalteFilesLaden


void EditierspalteEventBoxClass::EditierspalteSpeichern()
{
 // Sicherstellen, dass ein Benutzerklick nicht die aktuelle Spaltenauswahl veraendert, solange die Lade-
 // aktionen laufen. Sonst wuerde die Situation unsynchron werden:
  EditierspalteEventBoxClass* EditierspalteAktuellSpeicher = EditierspalteAktuell;
 // Umsortiertheit testen und warnen:  
 if (GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier])
  {
   Gtk::MessageDialog *SpeicherDialog = 
    new Gtk::MessageDialog(_("Really wanna save?"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
   SpeicherDialog->set_secondary_text
    (_("Column has been resorted but files not renamed.\nSaving discards this premliminary work."));  
   if (SpeicherDialog->run() == Gtk::RESPONSE_NO)
    {
     delete SpeicherDialog;
     return;
    }
   else
    {
     delete SpeicherDialog;
    }
  } 
 // Einen Anschluss zum Filesystem herstellen:
 FilehandlingClass FileSystem;
 // Auf das leere Verzeichnis testen. Ggf. zuerst ein Verzeichnis bestimmen und gleich anlegen lassen:  
 if (EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier] == _("void"))
  {
   try
    {
     std::string Verzeichnis = FileSystem.FilehandlingVerzeichnisAuswaehlen();
     EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = Verzeichnis;
     // Das aktualisierte Verzeichnis in die Datenfelder schreiben:
     for(ElementEventBoxClass* Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
         Zeiger != EditierspalteAktuellSpeicher->Ende;
         Zeiger = Zeiger->ElementDanach)
      {
       Zeiger->ElementUeberschreibeVerzeichnisNeu(Verzeichnis);
      }
     // Spaltennamen neu anzeigen:
     EditierspalteAktuellSpeicher->EditierspalteLabel.set_text(Verzeichnis);
    }
   catch(...)
    {
     std::cerr << "EditierspalteSpeichern: Cancellation by user." << std::endl;
     return;
    }
  }
 // Jetzt ist es Zeit, den Fortschrittsbalken zu zeigen:
 FortschrittDialogClass* FortschrittBalkenZeiger = new FortschrittDialogClass("PickSort", _("Saving files..."));
 FortschrittBalkenZeiger->SetFortschrittAnzeige(0.0);
 // Die Anzahl der Elemente zaehlen fuer den Fortschrittsbalken:
 ElementEventBoxClass* Zeiger;
 unsigned int FileAnzahl = 0;
 for (Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
      Zeiger != EditierspalteAktuellSpeicher->Ende;
      Zeiger = Zeiger->ElementDanach, ++FileAnzahl);
 // Dann die Elemente abspeichern:
 unsigned int Zaehler = 0;
 for (Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
      Zeiger != EditierspalteAktuellSpeicher->Ende;
      Zeiger = Zeiger->ElementDanach, ++Zaehler)
  {
   if (!FileSystem.FilehandlingElementSpeichern(Zeiger)) 
    {
     break;
    }
   Zeiger->SetElementUmbenannt(false);
   Zeiger->SetElementUmbewertet(false);
   FortschrittBalkenZeiger->SetFortschrittAnzeige((float)Zaehler / FileAnzahl);   
   if (FortschrittBalkenZeiger->GetFortschrittEndeAnforderung())
    {
     break;
    }
  }
 delete FortschrittBalkenZeiger;
 if (Zeiger != EditierspalteAktuellSpeicher->Ende)
  {
   Gtk::MessageDialog *SpeicherfehlerDialog =
    new Gtk::MessageDialog(_("Saving failed!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
   SpeicherfehlerDialog->set_secondary_text
    (_("Either due to problems with filesystem or triggered by user cancellation."));
   SpeicherfehlerDialog->run();
   delete SpeicherfehlerDialog;   
  }
 else
  {
   // Spaltenzustandsvariablen entsprechend dem erfolgreichen Speichern setzen:   
   GenerellesEditierspalteHingeschoben[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;
   GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;
  }
}  // EditierspalteSpeichern


void EditierspalteEventBoxClass::EditierspalteBenennen()
{
 // Sicherstellen, dass ein Benutzerklick nicht die aktuelle Spaltenauswahl veraendert, solange die Lade-
 // aktionen laufen. Sonst wuerde die Situation unsynchron werden:
 EditierspalteEventBoxClass* EditierspalteAktuellSpeicher = EditierspalteAktuell; 
 // Extra Fensterchen mit der Eingabezeile:
 Gtk::Entry* Eingabezeile = Gtk::manage(new Gtk::Entry());
 Eingabezeile->set_activates_default(true);
 Eingabezeile->set_width_chars(20); 
 Gtk::MessageDialog* FilenameEditieren =
  new Gtk::MessageDialog(_("Insert collection identifier name"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL, false);
 FilenameEditieren->set_secondary_text(_("Pattern of automatically generated filenames: identifier12345.jpg"));
 FilenameEditieren->get_message_area()->add(*Eingabezeile);
 FilenameEditieren->set_default_response(Gtk::RESPONSE_OK);
 FilenameEditieren->show_all();
 if (FilenameEditieren->run() == Gtk::RESPONSE_OK)
  {
   // Linux-geeigneten Sammelnamen generieren:
   std::string Sammelname = Glib::filename_from_utf8(Eingabezeile->get_buffer()->get_text());
   delete FilenameEditieren;
   // Den Fortschrittsbalken zeigen:
   FortschrittDialogClass* FortschrittBalkenZeiger = new FortschrittDialogClass("PickSort", _("Renaming files..."));
   FortschrittBalkenZeiger->SetFortschrittAnzeige(0.0);
   // Anzahl der Elemente in der Spalte zaehlen fuer den Fortschrittsbalken:
   ElementEventBoxClass* Zeiger;
   guint FileAnzahl = 1;
   for (Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
        Zeiger != EditierspalteAktuellSpeicher->Ende;
        Zeiger = Zeiger->ElementDanach, ++FileAnzahl);
   guint Zaehler = 1;
   gchar ZahlString[6];
   std::string FilenameNeu;
   for (Zeiger = EditierspalteAktuellSpeicher->Start->ElementDanach;
        Zeiger != EditierspalteAktuellSpeicher->Ende;
        Zeiger = Zeiger->ElementDanach, ++Zaehler)
    {
     g_snprintf(ZahlString, 6, "%05u", Zaehler);
     FilenameNeu = Sammelname + ZahlString + ".jpg";
     Zeiger->ElementUeberschreibeFilenameNeu(FilenameNeu);
     Zeiger->SetElementUmbenannt(true);
     // Fortschrittsbalken aktualisieren:
     FortschrittBalkenZeiger->SetFortschrittAnzeige((float)Zaehler / FileAnzahl);   
     if (FortschrittBalkenZeiger->GetFortschrittEndeAnforderung())
      {
       break;
      }
    } 
   delete FortschrittBalkenZeiger;
  }
 else
  {
   delete FilenameEditieren;   
  }
 // Umsortieren fuer diese Spalte zuruecksetzen
 GenerellesEditierspalteUmsortiert[EditierspalteAktuellSpeicher->EditierspalteNummerHier] = false;
}  // EditierspalteBenennen


void EditierspalteEventBoxClass::EditierspalteFiltern()
{
 // Sicherstellen, dass ein Benutzerklick nicht die aktuelle Spaltenauswahl veraendert, solange die Lade-
 // aktionen laufen. Sonst wuerde die Situation unsynchron werden:
 EditierspalteEventBoxClass* EditierspalteAktuellSpeicher = EditierspalteAktuell;
 // Extra Fensterchen mit der Bewertungsfilter-Eingabe, darin zwei Bewertungsgruppen platzieren:
 // Der jeweils erste Button ist nur als Platzhalter fuer Bewertung 0 gedacht. Er wird nicht angezeigt.
 Gtk::Grid FilterGridVon, FilterGridBis;
 Gtk::RadioButton FilterButtonVon[6], FilterButtonBis[6]; 
 Gtk::RadioButton::Group FilterButtonGroupVon = FilterButtonVon[0].get_group();
 Gtk::RadioButton::Group FilterButtonGroupBis = FilterButtonBis[0].get_group();
 for (int Zaehler = 1; Zaehler <= 5; ++Zaehler)
  {
   FilterButtonVon[Zaehler].set_group(FilterButtonGroupVon);
   FilterGridVon.attach(FilterButtonVon[Zaehler], Zaehler, 1, 1, 1);    
   FilterButtonBis[Zaehler].set_group(FilterButtonGroupBis);   
   FilterGridBis.attach(FilterButtonBis[Zaehler], Zaehler, 1, 1, 1);    
  }
 FilterButtonVon[0].set_active(true);
 FilterButtonBis[5].set_active(true);
 // Das Dialogfenster einrichten:
 Gtk::MessageDialog* FilterAuswaehlen =
  new Gtk::MessageDialog(_("Define column filter"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL, false);
 FilterAuswaehlen->set_secondary_text(_("Ratings from ... to (inclusive)"));
 FilterAuswaehlen->get_message_area()->add(FilterGridVon);
 FilterAuswaehlen->get_message_area()->add(FilterGridBis); 
 FilterAuswaehlen->show_all();
 if (FilterAuswaehlen->run() == Gtk::RESPONSE_OK)
  {   
   delete FilterAuswaehlen;
   // Benutzereingaben so richten, dass Filterung immer irgendwie funktioniert:
   short FilterVon, FilterBis, FilterMerker;
   for (FilterVon = 0; FilterVon < 5; ++FilterVon)
    {
     if (FilterButtonVon[FilterVon].get_active())
      {
       break;
      }
    } 
   for (FilterBis = 5; FilterBis > 0; --FilterBis)
    {
     if (FilterButtonBis[FilterBis].get_active())
      {
       break;
      }
    }
   if (FilterBis < FilterVon)
    {
     FilterMerker = FilterBis;
     FilterBis = FilterVon;
     FilterVon = FilterMerker;
    }
   // In allen Elementen der Spalte die Sichtbarkennung setzen, dabei fuer den Fortschrittsbalken zaehlen:
   unsigned int FileAnzahl = 0;
   unsigned int Zaehler = 0;
   ElementEventBoxClass* Zeiger;
   for (Zeiger = EditierspalteAktuell->Start->ElementDanach;
        Zeiger != EditierspalteAktuell->Ende;
        Zeiger = Zeiger->ElementDanach, ++FileAnzahl)
    {
     if ((Zeiger->GetElementBewertungNeu() >= FilterVon) && (Zeiger->GetElementBewertungNeu() <= FilterBis))
      {
       Zeiger->SetElementSichtbar(true);
      }
     else
      {
       Zeiger->SetElementSichtbar(false);
      }
    }
   // Fortschrittsbalken zeigen:
   FortschrittDialogClass* FortschrittBalkenZeiger
    = new FortschrittDialogClass("PickSort", _("Filtering files by ratings..."));
   FortschrittBalkenZeiger->SetFortschrittAnzeige(0.0);  
   // Aktuelle Anzeige im Spaltengrid loeschen und neu mit den gefilterten Elementen befuellen,
   // dazu altes Grid loeschen:
   delete EditierspalteAktuellSpeicher->EditierspalteGrid;
   // Neues Grid anlegen; mit manage() z.B wegen des Programmendes:
   EditierspalteAktuellSpeicher->EditierspalteGrid = Gtk::manage(new Gtk::Grid());
   EditierspalteAktuellSpeicher->EditierspalteRahmen.add(*(EditierspalteAktuellSpeicher->EditierspalteGrid));
   // Dann die gefilterten Elemente neu in das neue Grid laden:
   unsigned int BildPosition = 0;
   for (Zeiger = EditierspalteAktuell->Start->ElementDanach;
        Zeiger != EditierspalteAktuell->Ende;
        Zeiger = Zeiger->ElementDanach, ++Zaehler)   
    {
     // Element ins Grid laden:
     if (Zeiger->GetElementSichtbar())
      {
       EditierspalteAktuellSpeicher->EditierspalteGrid->attach(*Zeiger, 1, ++BildPosition, 1, 1);    
       // Fortschritt anzeigen:
       FortschrittBalkenZeiger->SetFortschrittAnzeige((float)Zaehler / FileAnzahl);  
       if (FortschrittBalkenZeiger->GetFortschrittEndeAnforderung()) break;
      }       
    }
   // Das Abschlusselement ins Grid laden: 
   EditierspalteAktuellSpeicher->EditierspalteGrid->attach
    (*(EditierspalteAktuellSpeicher->Ende), 1, ++BildPosition, 1, 1);      
   EditierspalteAktuellSpeicher->show_all();
   delete FortschrittBalkenZeiger;         
   if ((FilterVon == 0) && (FilterBis == 5))
    {
     EditierspalteAktuellSpeicher->EditierspalteLabel.set_text
      (EditierspaltenNamen[EditierspalteAktuellSpeicher->EditierspalteNummerHier]); 
    }
   else
    {
     EditierspalteAktuellSpeicher->EditierspalteLabel.set_text(_("filtered")); 
    }
  }
 else
  {
   delete FilterAuswaehlen;
  }
}  // EditierspalteFiltern


void EditierspalteEventBoxClass::EditierspalteSortieren()
{
 // Sicherstellen, dass ein Benutzerklick nicht die aktuelle Spaltenauswahl veraendert, solange die Lade-
 // aktionen laufen. Sonst wuerde die Situation unsynchron werden:
 EditierspalteEventBoxClass* EditierspalteAktuellSpeicher = EditierspalteAktuell;
 // Einlesen, ob nach aufsteigendem Name oder nach aufsteigendem Datum sortiert werden soll:
 Gtk::RadioButton AuswahlButtonName, AuswahlButtonDatum;
 Gtk::RadioButton::Group AuswahlButtonGroup = AuswahlButtonName.get_group();
 AuswahlButtonDatum.set_group(AuswahlButtonGroup);
 AuswahlButtonName.set_active(true);
 Gtk::Label AuswahlLabelName(_("Name"));
 Gtk::Label AuswahlLabelDatum(_("Date"));
 Gtk::Grid AuswahlGrid; 
 AuswahlGrid.attach(AuswahlButtonName, 1, 1, 1, 1);
 AuswahlGrid.attach(AuswahlButtonDatum, 1, 2, 1, 1);
 AuswahlGrid.attach(AuswahlLabelName, 2, 1, 1, 1);
 AuswahlGrid.attach(AuswahlLabelDatum, 2, 2, 1, 1);
 Gtk::MessageDialog* SortierkriteriumAuswaehlen =
  new Gtk::MessageDialog(_("Choose sorting criterion"), false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL, false);
 SortierkriteriumAuswaehlen->set_secondary_text(_("Column will be sorted in ascending order of criterion."));
 SortierkriteriumAuswaehlen->get_message_area()->add(AuswahlGrid);
 SortierkriteriumAuswaehlen->show_all();
 if (SortierkriteriumAuswaehlen->run() == Gtk::RESPONSE_OK)
  {
   // Den Sortieralgorithmus anwenden, nach Name und Datum unterscheiden:
   SortierAlgorithmus(EditierspalteAktuell, AuswahlButtonName.get_active());
   // Aktuelle Anzeige im Spaltengrid loeschen und neu mit den gefilterten Elementen befuellen,
   // dazu altes Grid loeschen:
   // Manuell deallokieren:
   delete EditierspalteAktuellSpeicher->EditierspalteGrid;
   // Neues Grid anlegen; mit manage() z.B wegen des Programmendes:   
   EditierspalteAktuellSpeicher->EditierspalteGrid = Gtk::manage(new Gtk::Grid());
   EditierspalteAktuellSpeicher->EditierspalteRahmen.add(*(EditierspalteAktuellSpeicher->EditierspalteGrid));
   // Dann die sortierten Elemente neu in das neue Grid laden:
   ElementEventBoxClass* Zeiger;
   unsigned int BildPosition = 1;
   for (Zeiger = EditierspalteAktuell->Start->ElementDanach;
        Zeiger != EditierspalteAktuell->Ende;
        Zeiger = Zeiger->ElementDanach)   
    {
     EditierspalteAktuellSpeicher->EditierspalteGrid->attach(*Zeiger, 1, ++BildPosition, 1, 1);    
    }       
   // Das Abschlusselement ins Grid laden: 
   EditierspalteAktuellSpeicher->EditierspalteGrid->attach
    (*(EditierspalteAktuellSpeicher->Ende), 1, ++BildPosition, 1, 1);      
   EditierspalteAktuellSpeicher->show_all(); 
  }
 delete SortierkriteriumAuswaehlen; 
}  // EditierspalteSortieren 


void EditierspalteEventBoxClass::SortierAlgorithmus(EditierspalteEventBoxClass* Editierspalte, bool IstName)
{
 // Natural Merge-Sort auf einer doppelt verketteten Liste:
 // Schritt 0: Schnellausstieg, wenn die Spalte leer sein sollte:
 if (Editierspalte->Start->ElementDanach == Editierspalte->Ende) return; 
 // Dann ist es Zeit fuer den Fortschrittsbalken:
 FortschrittDialogClass* FortschrittBalkenZeiger
  = new FortschrittDialogClass("PickSort", _("Sorting files by name / date..."));
 FortschrittBalkenZeiger->SetFortschrittAnzeige(0.0);   
 // Schritt 1: Sortiere ansteigende Elementfolgen in Blöcke:
 bool ReiheSteigt;
 ElementEventBoxClass* Lauf1;
 ElementEventBoxClass* Lauf2;
 // Blockstruktur in schneller Low-Level-C-Syntax einrichten:
 struct BlockStructure
  {
   BlockStructure* BlockDavor;
   BlockStructure* BlockDanach;
   ElementEventBoxClass* Element;
  };
 BlockStructure* BlockListeStart = new BlockStructure; 
 BlockStructure* BlockListeEnde = new BlockStructure; 
 BlockListeStart->BlockDavor = NULL;
 BlockListeStart->BlockDanach = BlockListeEnde;
 BlockListeStart->Element = Editierspalte->Start;
 BlockListeEnde->BlockDanach = NULL;
 BlockListeEnde->BlockDavor = BlockListeStart;
 BlockListeEnde->Element = Editierspalte->Ende;
 // Der Iterator in der Blockstruktur:
 BlockStructure* BlockListeZeiger;
 // Fange mit dem zweiten Element an, es existiert nach Schritt 0 immer und ist sicher nicht das Ende:
 Lauf1 = Editierspalte->Start->ElementDanach;
 // Wegen der notwendigen Symmetrie muss das erste richtige Element gleich der Anfang eines Blockes sein:
 BlockListeZeiger = new BlockStructure;
 BlockListeZeiger->BlockDanach = BlockListeEnde;
 BlockListeZeiger->BlockDavor = BlockListeStart;
 BlockListeEnde->BlockDavor = BlockListeZeiger;
 BlockListeStart->BlockDanach = BlockListeZeiger;
 BlockListeZeiger->Element = Lauf1;
 // Die Anfangs-Steigungsbedingung ist immer das Fallen:
 ReiheSteigt = false; 
 // Zaehlvariablen fuer den Forschrittsbalken:
 unsigned int BlockSumme = 0;
 unsigned int BlockZaehler = 0;
 // Durchsuche bis zum letzten Element vor dem Ende. Das Endeelement wird also genau ein Mal mitverglichen:
 do 
  {
   // Vergleichsoperator <= damit die Sortierung robust bleibt: Gleiche Elemente behalten ihre Reihenfolge:
   if (IstName ? Lauf1->GetElementFilenameNeu() <= Lauf1->ElementDanach->GetElementFilenameNeu() : Lauf1->GetElementDatum() <= Lauf1->ElementDanach->GetElementDatum())
   // dann steigt die Folge zum direkt nachfolgenden Element - daher Steuervariable auf true setzen:
    {
     ReiheSteigt = true;
    }
   else
   // Wenn die Folge also nicht steigt:
    {
     if (ReiheSteigt)
     // - beim ersten fallenden Element: Nur vermerken in Steuervariable und Blockliste:
      {
       BlockListeZeiger = new BlockStructure;
       BlockListeZeiger->BlockDanach = BlockListeEnde;
       BlockListeZeiger->BlockDavor = BlockListeEnde->BlockDavor;
       BlockListeEnde->BlockDavor->BlockDanach = BlockListeZeiger;
       BlockListeEnde->BlockDavor = BlockListeZeiger;
       BlockListeZeiger->Element = Lauf1->ElementDanach;
       ReiheSteigt = false;     
       ++BlockSumme;
      }
     else
     // - beim zweiten fallenden Element in Folge: Elemente tauschen,
     // Blockliste und Zeiger aktualisieren:
      {
       // Zeiger auf zu editierende Elemente justieren:
       Lauf2 = Lauf1;
       Lauf1 = Lauf1->ElementDanach; 
       // Aussenzeiger auf die Elemente tauschen:
       Lauf2->ElementDavor->ElementDanach = Lauf1;
       Lauf1->ElementDanach->ElementDavor = Lauf2;
       // Innenzeiger auf die Anschlusselemente tauschen:
       Lauf1->ElementDavor = Lauf2->ElementDavor;
       Lauf2->ElementDanach = Lauf1->ElementDanach;
       // Innenzeiger auf die zwei betrachteten Elemente tauschen:
       Lauf1->ElementDanach = Lauf2;
       Lauf2->ElementDavor = Lauf1;
       // Lauf1 ist nun das erste, da kleinere Element und bekommt den Blockzeiger:
       BlockListeZeiger->Element = Lauf1;     
       ReiheSteigt = true;
       // Lauf1 zeigt nun auf das erste der beiden verglichenen Elemente. Am Ende der Schleife wird zum
       // zweiten Element weitergeschaltet und der Vergleich mit dessen Nachfolger in der Folgeschleife
       // dann gemacht.
      }      
    }
   // Zum naechsten Element weiterschalten:
   Lauf1 = Lauf1->ElementDanach;
  }
 while (Lauf1 != Editierspalte->Ende);  
 // Schritt 2: Fasse Bloecke ueber wiederholte Merge-Schritte zusammen:
 // Wenn Blockliste nur einen Block enthaelt, dann ist bereits fertig sortiert; springe also direkt heraus.
 // Aus Schritt 1 gibt es dann genau einen Block zwischen Blockstart und Blockende,
 // auf diesen zeigt der BlockZeiger:
 ElementEventBoxClass* LaufZeiger;
 ElementEventBoxClass* Lauf1Ende;
 ElementEventBoxClass* Lauf2Ende;
 ElementEventBoxClass* MergeStart;
 ElementEventBoxClass* MergeEnde;   
 // Beim Loeschen eines Elementes braucht man immer einen zweiten Zeiger:
 BlockStructure* BlockListeHilfsZeiger;
 // Blockzeiger auf den ersten Block setzen:
 BlockListeZeiger = BlockListeStart->BlockDanach;
 // In jeder Mergerunde werden Bloecke vereinigt. Die Runden laufen so lange, bis nur noch 
 // ein Block da ist, der durch Block-Start, den ersten Block und Block-Ende gekennzeichnet ist.
 // Abweisende Schleife, weil das auch der Test vor dem Einstieg in die Schleife ist:
 while (BlockListeStart->BlockDanach != BlockListeEnde->BlockDavor)
  {
   // In der Mergerunde werden immer der aktuelle und der Nachfolgeblock vereinigt. Der BlockZeiger des
   // Nachfolgeblockes wird gleich geloescht, am Ende dann der Blockzeiger auf den Nachfolger gesetzt.
   // Ausserdem muss am Ende der Mergerunde der BlockZeiger wieder auf Block-Start gesetzt werden:
   do
    {
     // Die LaufZeiger entsprechend setzen:
     Lauf1 = BlockListeZeiger->Element;  
     Lauf2 = BlockListeZeiger->BlockDanach->Element;
     Lauf1Ende = BlockListeZeiger->BlockDanach->Element->ElementDavor;
     Lauf2Ende = BlockListeZeiger->BlockDanach->BlockDanach->Element->ElementDavor;
     // Zwischen MergeStart und MergeEnde wird die zu mergende Kette umgeknüpft.
     // Die Elemente bei MergeStart und MergeEnde werden selbst nicht umgeknuepft, sie sind die Anker
     // beim Einfuegen dann. Es ligen die zu vermengenden Elemente in zwei Teilketten vor. Die Anfangs-
     // elemente der Teilketten haben keine sinnvollen Vorgaenger, die Endeelemente keine Nachfolger:
     MergeStart = BlockListeZeiger->Element->ElementDavor;
     // Immer zwei Bloecke werden zu einem vermengt:
     MergeEnde = BlockListeZeiger->BlockDanach->BlockDanach->Element;
     MergeStart->ElementDanach = MergeEnde;
     MergeEnde->ElementDavor = MergeStart;
     // Den Nachfolger des BlockZeigers gleich herausnehmen, denn die beiden Blöcke werden ja vereint:
     BlockListeHilfsZeiger = BlockListeZeiger->BlockDanach;
     BlockListeZeiger->BlockDanach = BlockListeHilfsZeiger->BlockDanach;
     BlockListeHilfsZeiger->BlockDanach->BlockDavor = BlockListeZeiger;
     delete BlockListeHilfsZeiger;
     ++BlockZaehler;
     FortschrittBalkenZeiger->SetFortschrittAnzeige((float)BlockZaehler / BlockSumme); 
     if (FortschrittBalkenZeiger->GetFortschrittEndeAnforderung())
      {
       // Allokierte Blockelemente deallokieren, dann zuruckespringen:
       do
        {
         BlockListeZeiger = BlockListeStart;
         BlockListeStart = BlockListeStart->BlockDanach;
         delete BlockListeZeiger;
        }
       while (BlockListeStart != BlockListeEnde);
       delete BlockListeStart;
       delete BlockListeEnde;
       // Und auch den Fortschrittsbalken aufraeumen:
       delete FortschrittBalkenZeiger;
       return;      
      }
     // Elementweiser Vergleich des aktuellen Blockes und des Nachfolgeblockes. Einsortieren in die
     // Merge-Struktur. Diese ist bereits in die Gesamtliste eingehaengt. Am Ende muss der aktuelle 
     // BlockZeiger neu justiert werden:
     for(;;)
      {
       if (IstName ? Lauf1->GetElementFilenameNeu() <= Lauf2->GetElementFilenameNeu() : Lauf1->GetElementDatum() <= Lauf2->GetElementDatum())
        {
         // Wenn die Lauf1-Schlange das letzte Element noch nicht erreicht hat:
         if (Lauf1 != Lauf1Ende)
          {
           // Lauf1 eine Stelle weiterschalten und das bisherige Element aushaengen:
           LaufZeiger = Lauf1;
           Lauf1 = Lauf1->ElementDanach;
           // Das ausgehaengte Element in die Mergeliste einknuepfen:
           LaufZeiger->ElementDanach = MergeEnde;
           LaufZeiger->ElementDavor = MergeEnde->ElementDavor;
           MergeEnde->ElementDavor->ElementDanach = LaufZeiger;
           MergeEnde->ElementDavor = LaufZeiger;
          }
         // Wenn die Lauf1-Schlange das letzte Element erreicht hat:
         else
          {
           // Letztes Lauf1-Element Umhängen und dann den Rest der Lauf2-Schlange anhängen:
           // Das letzte Lauf1-Element in die neue Liste einknuepfen:
           Lauf1->ElementDanach = MergeEnde;
           Lauf1->ElementDavor = MergeEnde->ElementDavor;
           MergeEnde->ElementDavor->ElementDanach = Lauf1;
           MergeEnde->ElementDavor = Lauf1;
           // Dann den Rest von Lauf2 anhaengen:
           Lauf2Ende->ElementDanach = MergeEnde;
           Lauf2->ElementDavor = MergeEnde->ElementDavor;
           MergeEnde->ElementDavor->ElementDanach = Lauf2;
           MergeEnde->ElementDavor = Lauf2Ende;
           // Jetzt sind alle Elemente der aktuellen Bloecke vereinigt:
           break;  // for(;;)
          }
        }
       // Wenn Lauf2->... < Lauf1->...:
       else  
        {
         // Wenn Lauf2-Schlange noch nicht das letzte Element erreicht hat:
         if (Lauf2 != Lauf2Ende)
          {
           // Lauf2 eine Stelle weiterschalten und das bisherige Element aushaengen:
           LaufZeiger = Lauf2;
           Lauf2 = Lauf2->ElementDanach;
           //Element selbst in die neue Liste einknuepfen:
           LaufZeiger->ElementDanach = MergeEnde;
           LaufZeiger->ElementDavor = MergeEnde->ElementDavor;
           MergeEnde->ElementDavor->ElementDanach = LaufZeiger;
           MergeEnde->ElementDavor = LaufZeiger;
          }
         // Wenn die Lauf2-Schlange das letzte Element erreicht hat:
         else
          {
           // Element Umhängen und dann den Rest der Lauf1-Schlange anhängen:
           // Element selbst in die neue Liste einknuepfen:
           Lauf2->ElementDanach = MergeEnde;
           Lauf2->ElementDavor = MergeEnde->ElementDavor;
           MergeEnde->ElementDavor->ElementDanach = Lauf2;
           MergeEnde->ElementDavor = Lauf2;
           // Dann den Rest von Lauf1 anhaengen:
           Lauf1Ende->ElementDanach = MergeEnde;
           Lauf1->ElementDavor = MergeEnde->ElementDavor;
           MergeEnde->ElementDavor->ElementDanach = Lauf1;
           MergeEnde->ElementDavor = Lauf1Ende;
           // Jetzt sind alle Elemente der aktuellen Bloecke vereinigt:
           break;  // for(;;)
          }
        }
      }  // for (;;)
     // Der BlockZeiger zeigt nach den Mergeoperationen in der Regel nicht auf das erste Element des
     // bislang betrachteten Doppelblockes. Daher muss der BlockZeiger neu gesetzt werden:
     BlockListeZeiger->Element = MergeStart->ElementDanach;
     // Der urspruenglich nächste Blockzeiger wurde ja geloescht,
     // jetzt also auf den aktuell naechsten Block springen: 
     BlockListeZeiger = BlockListeZeiger->BlockDanach;
    }
   // Es muessen allgemein immer zwei Bloecke noch vorhanden sein, um einen Mergelauf machen zu koennen.
   // Die Mergerunde kann so enden, dass beim Weiterschalten des BlockListenZeigers dieser dann auf dem
   // Blockelement vor BlockEnde oder direkt auf BlockEnde zu stehen kommt. Daher sind zwei Abbruch-
   // bedingungen zu testen:
   while((BlockListeZeiger->BlockDanach != BlockListeEnde) & (BlockListeZeiger != BlockListeEnde));
   // Nach der inneren  Mergerunde den Zeiger in der aeusseren Mergerunde wieder auf den ersten Block setzen:
   BlockListeZeiger = BlockListeStart->BlockDanach;   
  }  // while (BlockListeStart->BlockDanach != BlockListeEnde->BlockDavor)
 // Die letzten allokierten Blockzeiger deallokieren
 delete BlockListeZeiger;
 delete BlockListeStart;
 delete BlockListeEnde;
 delete FortschrittBalkenZeiger;
}  // SortierAlgorithmus


void EditierspalteEventBoxClass::EditierspaltePraesentieren()
{
 // Alle Verzeichnis- und Filenamen der Elemente der Spalte
 // ab dem aktiven Element Element in ein Verzeichnisfile schreiben:
 ElementEventBoxClass* StartElement = EditierspalteAktuell->Start->GetElement1Gewaehlt();
 ElementEventBoxClass* EndeElement = EditierspalteAktuell->Start->GetElement2Gewaehlt(); 
 // Wenn in eine Spalte, aber nicht in ein Element geklickt wurde...:
 if (StartElement == NULL)
  {
   // ...dann versuche es, die Spalte vom ersten Element an zu zeigen:
   StartElement = EditierspalteAktuell->Start->ElementDanach;
 // Die Spalte koennte dann aber auch leer sein:
   if (StartElement == EditierspalteAktuell->Ende)
    {
     return;
    } 
  }
 // Wenn kein Endeelement angeklickt wurde:
 if (EndeElement == NULL)
  {
   // ... dann nehme das Ende der Spalte;
   EndeElement = EditierspalteAktuell->Ende->ElementDavor;
  }
 // Eine temporaere Text-Datei anlegen, evtl. noch vorhandene Datei wird einfach ueberschrieben:
 std::ofstream BilderFilenamenDatei("./PickSortProFiles.txt", std::ios::trunc);
 if (!BilderFilenamenDatei)
  {
   Gtk::MessageDialog *SpeicherfehlerDialog =
    new Gtk::MessageDialog(_("Unable to create PickSortPro control file!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
   SpeicherfehlerDialog->set_secondary_text
    (_("There are problems with the filesystem."));
   SpeicherfehlerDialog->run(); 
   delete SpeicherfehlerDialog;   
   return;
  }
 // Diese Datei dann befuellen und schliessen:
 std::string BilderFilename; 
 for (ElementEventBoxClass* Element = StartElement;
      Element != EndeElement->ElementDanach;
      Element = Element->ElementDanach)
  {
   BilderFilename = Element->GetElementVerzeichnisAlt() + "/" + Element->GetElementFilenameAlt();
   BilderFilenamenDatei << BilderFilename << std::endl;
  }
 // File sofort schliessen, so dass das aufzurufende Programm gleich zugreifen kann:
 BilderFilenamenDatei.close(); 
 // Praesentationsroutine mit dem Pfad und dem Filename des Verzeichnisfiles aufrufen:
 int Fehler = system("./picksortpro ./PickSortProFiles.txt");
 if (Fehler)
  {
   Gtk::MessageDialog *SpeicherfehlerDialog =
    new Gtk::MessageDialog(_("System call failed!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
   SpeicherfehlerDialog->set_secondary_text
    (_("Cannot start PickSortPro."));
   SpeicherfehlerDialog->run();
   delete SpeicherfehlerDialog;   
   return;
  }  
}  // EditierspaltePraesentieren


void EditierspalteEventBoxClass::EditierspalteElementRechtsDrehen()
{
 EditierspalteAktuell->Start->ElementRechtsDrehen();
}  // EditierspalteElementRechtsDrehen


void EditierspalteEventBoxClass::EditierspalteElementLinksDrehen()
{
 EditierspalteAktuell->Start->ElementLinksDrehen();
}  // EditierspalteElementLinksDrehen


void EditierspalteEventBoxClass::EditierspalteElementBetrachten()
{
 EditierspalteAktuell->Start->ElementBetrachten();
}  // EditierspalteElementBetrachten


void EditierspalteEventBoxClass::EditierspalteElementEditieren()
{
 EditierspalteAktuell->Start->ElementEditieren();
}  // EditierspalteElementEditieren


bool EditierspalteEventBoxClass::GetUmbenanntUmbewertetGedreht()
{
 for(ElementEventBoxClass* Zeiger = Start->ElementDanach;
     Zeiger != Ende;
     Zeiger = Zeiger->ElementDanach)
  {
   if (Zeiger->GetElementUmbenannt() || Zeiger->GetElementUmbewertet() || (Zeiger->GetElementDrehung() != 0))
    {
     return true;
    }
  }
 return false;
}  // GetUmbenanntUmbewertet


bool EditierspalteEventBoxClass::GetSpalteIstLeer()
{
 if (Start->ElementDanach == Ende)
  {
   return true;
  }
 else
  {
   return false;
  }
}  // GetSpalteIstLeer


bool EditierspalteEventBoxClass::EditierspalteButtonPressEventHandler
 (GdkEventButton* EditierspalteButtonPressEvent)
{
 EditierspalteAktuell->override_background_color(GenerellesSpalteFarbeNormal);
 EditierspalteAktuell = this;
 EditierspalteAktuell->override_background_color(GenerellesSpalteFarbeGewaehlt);
 // Da Signalhandler ganz vorne eingefuegt ist, X-Window-Signal durch "return false" weiterreichen:
 return false;
}  // EditierspalteButtonPressEventHandler


/*
 * PickSortEditierspalte.cpp *********************************************************************************
 */

