/*
 *************************************************************************************************************
 * PickSortElement.cpp     
 *
 * Die Struktur, die das Bild und einige Bildinformationen kapselt.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortElement.h"


ElementEventBoxClass* ElementEventBoxClass::Element1Gewaehlt = NULL;
ElementEventBoxClass* ElementEventBoxClass::Element2Gewaehlt = NULL;
std::vector<Gtk::TargetEntry> ElementEventBoxClass::TargetVektor;
bool ElementEventBoxClass::TargetVektorInitialisiert = false;
unsigned short ElementEventBoxClass::NrSpalteAktuellGewaehlt = 0;


ElementEventBoxClass::ElementEventBoxClass(const std::string& Filename,
                                           const std::string& Fotografierdatum,
                                           bool Sichtbarkeit,
                                           unsigned short SpaltenNrHier)
{
 // Element mit dem X-Window verbinden:
 set_events(Gdk::BUTTON_PRESS_MASK);
 add_events(Gdk::ENTER_NOTIFY_MASK);
 // Callback-Funktionen:
 signal_button_press_event().connect
  (sigc::mem_fun(*this, &ElementEventBoxClass::ElementButtonPressHandler), false);
 signal_enter_notify_event().connect
  (sigc::mem_fun(*this, &ElementEventBoxClass::ElementEnterNotifyHandler), false);
 signal_leave_notify_event().connect
  (sigc::mem_fun(*this, &ElementEventBoxClass::ElementLeaveNotifyHandler), false);
 // Drag und Drop:
 // Targettyp nur einmal vereinbaren:
 if (!TargetVektorInitialisiert)
  {
   TargetVektorInitialisiert = true;
   // Drag- und Drop-Typ festlegen:
   TargetVektor.push_back(Gtk::TargetEntry("PickSortElement", Gtk::TARGET_SAME_APP, 1961));
  } // Drag und Drop:
 drag_source_set	(TargetVektor, Gdk::BUTTON1_MASK, Gdk::ACTION_MOVE);
 drag_dest_set(TargetVektor, Gtk::DEST_DEFAULT_ALL, Gdk::ACTION_MOVE);
 signal_drag_drop().connect(sigc::mem_fun(*this, &ElementEventBoxClass::ElementDragDropHandler), false);
 // Bild: Briefmarke:
 ElementImage.set(GenerellesPixelbufferSW);
 ElementImage.set_padding(5, 5);
 // Filename:
 ElementLabelFilename.set_label("PickSort  v0.4  08/2014");
 ElementLabelFilename.set_ellipsize(Pango::ELLIPSIZE_START);
 ElementLabelFilename.override_color(GenerellesElementSchrift);
 // Dies positioniert dann die ganze Beschriftungsspalte neben dem Bild in der verbleibenden Spaltenbreite:
 ElementLabelFilename.set_hexpand(true);
 ElementLabelFilename.set_halign	(Gtk::ALIGN_END);	
 ElementLabelFilename.set_margin_right(5);
 // Infospalte zusammenfassen:
 ElementLabelGrid.attach(ElementLabelFilename, 1, 1, 1, 1);
 // Schriftfeld in der verbleibenden Breite zentrieren:
 ElementLabelAlignment.add(ElementLabelGrid);
 ElementLabelAlignment.set(Gtk::ALIGN_FILL, Gtk::ALIGN_CENTER, 1.0, 0.0);
 // Element zusammenfassen:
 ElementGrid.attach(ElementImage, 1, 1, 1, 1);
 ElementGrid.attach(ElementLabelAlignment, 2, 1, 1, 1);
 add(ElementGrid); 
 // Element mit der eigenen Farbe einfaerben:
 override_background_color(GenerellesElementFarbeNormal);
 show_all(); 
 // Drag und Drop: Targettyp nur einmal vereinbaren:
 if (!TargetVektorInitialisiert)
  {
   TargetVektorInitialisiert = true;
   // Drag- und Drop-Typ festlegen:
   TargetVektor.push_back(Gtk::TargetEntry("PickSortElement", Gtk::TARGET_SAME_APP, 1961));
  }
 // Kontrollvariable fuer Ausgewaehlt-Sein:
 ElementGewaehlt = false;
 // Daten einfuellen:
 VerzeichnisAlt = VerzeichnisNeu = "";
 FilenameAlt = FilenameNeu = Filename;
 Datum = Fotografierdatum; 
 BewertungAlt = BewertungNeu = 0;
 Drehung = 0;
 Sichtbar = Sichtbarkeit;
 NrSpalteHier = SpaltenNrHier;
 StartEndeElement = true;
 ElementUmbenannt = false;
 ElementUmbewertet = false;
 // Verkettung vorbereiten:
 ElementDavor = ElementDanach = NULL;
}  // ElementEventBoxClass


ElementEventBoxClass::ElementEventBoxClass(const std::string& Verzeichnisname,
                                           const std::string& Filename, 
                                           const std::string& Fotografierdatum,
                                           short Bewertung,
                                           bool Sichtbarkeit,
                                           Glib::RefPtr<Gdk::Pixbuf> Pixelbuffer,
                                           unsigned short SpaltenNrHier,
                                           int BildDrehung)
{
 // Element mit dem X-Window verbinden:
 set_events(Gdk::BUTTON_PRESS_MASK);
 add_events(Gdk::ENTER_NOTIFY_MASK);
 // Callback-Funktionen:
 // ... das ganze Element betreffend:
 signal_button_press_event().connect
  (sigc::mem_fun(*this, &ElementEventBoxClass::ElementButtonPressHandler), false);
 signal_enter_notify_event().connect
  (sigc::mem_fun(*this, &ElementEventBoxClass::ElementEnterNotifyHandler), false);
 signal_leave_notify_event().connect
  (sigc::mem_fun(*this, &ElementEventBoxClass::ElementLeaveNotifyHandler), false);
 // Drag und Drop:
 // Targettyp nur einmal vereinbaren:
 if (!TargetVektorInitialisiert)
  {
   TargetVektorInitialisiert = true;
   // Drag- und Drop-Typ festlegen:
   TargetVektor.push_back(Gtk::TargetEntry("PickSortElement", Gtk::TARGET_SAME_APP, 1961));
  }
 drag_source_set	(TargetVektor, Gdk::BUTTON1_MASK, Gdk::ACTION_MOVE);
 drag_dest_set(TargetVektor, Gtk::DEST_DEFAULT_ALL, Gdk::ACTION_MOVE);
 signal_drag_drop().connect(sigc::mem_fun(*this, &ElementEventBoxClass::ElementDragDropHandler), false);
 // Grafikelement:
 ElementImage.set(Pixelbuffer);
 ElementImage.set_padding(5, 5);  
 // Filename:
 ElementLabelFilename.set_label(Glib::filename_to_utf8(Filename));
 ElementLabelFilename.set_ellipsize(Pango::ELLIPSIZE_START);
 ElementLabelFilename.override_color(GenerellesElementSchrift);
 // Dies fuellt dann die ganze Beschriftungsspalte neben dem Bild in der verbleibenden Spaltenbreite.
 // Alle ohne h- oder v-expand qualifizierten Elemente bleiben klein und auf der linken Seite.
 // Das am weitesten innen geschachtelte Element entscheidet offenbar:
 ElementLabelFilename.set_hexpand(true);
 ElementLabelFilename.set_halign	(Gtk::ALIGN_END);	
 ElementLabelFilename.set_margin_right(5);
 // Aufnahmedatum:
 ElementLabelAufnahmedatum.set_label(Fotografierdatum);
 ElementLabelAufnahmedatum.set_ellipsize(Pango::ELLIPSIZE_START); 
 ElementLabelAufnahmedatum.override_color(GenerellesElementSchrift);
 ElementLabelAufnahmedatum.set_halign	(Gtk::ALIGN_END);	
 ElementLabelAufnahmedatum.set_margin_right(5);
 // Bewertung:
 // Der erste Button ist nur als Platzhalter fuer Bewertung 0. Er wird nicht angezeigt und auf ihn
 // wird auch kein Signal eingehaengt:
 Gtk::RadioButton::Group ElementBewertungGroup = ElementBewertungButton[0].get_group();
 for (int Zaehler = 1; Zaehler <= 5; ++Zaehler)
  {
   ElementBewertungButton[Zaehler].set_group(ElementBewertungGroup);
   // Signale der Bewertungsbuttons verbinden:   
   ElementBewertungButton[Zaehler].signal_clicked().connect
    (sigc::bind<int>(sigc::mem_fun(*this, &ElementEventBoxClass::ElementBewertungsButtonHandler), Zaehler));
   ElementBewertungGrid.attach(ElementBewertungButton[Zaehler], Zaehler, 1, 1, 1);     
  }
 ElementBewertungGrid.set_halign(Gtk::ALIGN_END);
 ElementBewertungGrid.set_margin_right(0);
 if (Bewertung <= 0)
  {
   // Dieses Element sieht man nicht, da nicht ins Grid geladen:
   ElementBewertungButton[0].set_active(true);
  }
 else
  {
   if (Bewertung > 5)
    {
     Bewertung = 5;
    }
   ElementBewertungButton[Bewertung].set_active(true);
  } 
 // Infospalte zusammenfassen:
 ElementLabelGrid.attach(ElementLabelFilename, 1, 1, 1, 1);
 ElementLabelGrid.attach(ElementLabelAufnahmedatum, 1, 2, 1, 1);
 ElementLabelGrid.attach(ElementBewertungGrid, 1, 3, 1, 1);
 // Schriftfeld in der verbleibenden Flaeche zentrieren:
 ElementLabelAlignment.add(ElementLabelGrid);
 ElementLabelAlignment.set(Gtk::ALIGN_FILL, Gtk::ALIGN_CENTER, 1.0, 0.0);
 // Element zusammenfassen:
 ElementGrid.attach(ElementImage, 1, 1, 1, 1);
 ElementGrid.attach(ElementLabelAlignment, 2, 1, 1, 1);
 add(ElementGrid);
 // Element mit der eigenen Farbe einfaerben:
 override_background_color(GenerellesElementFarbeNormal);
 show_all();
 // Kontrollvariable fuer Ausgewaehlt-Sein:
 ElementGewaehlt = false;
 // Daten einfuellen:
 VerzeichnisAlt = VerzeichnisNeu = Verzeichnisname;
 FilenameAlt = FilenameNeu = Filename;
 Datum = Fotografierdatum; 
 BewertungAlt = BewertungNeu = Bewertung;
 Drehung = BildDrehung;
 Sichtbar = Sichtbarkeit;
 NrSpalteHier = SpaltenNrHier;
 StartEndeElement = false; 
 ElementUmbenannt = false;
 ElementUmbewertet = false;
 // Verkettung vorbereiten:
 ElementDavor = ElementDanach = NULL;
}  //   ElementEventBoxClass  


ElementEventBoxClass::~ElementEventBoxClass()
{
 // Beim Loeschen einer Spalte durch ...|neu testen, ob noch mindestens ein Element ausgewaehlt war.
 // Wenn dem so ist, dann die static-Elemente-Zeiger zuruecksetzen, weil sonst, beim ersten Anklicken
 // eines anderen Elementes einer anderen Spalte es dazu kommt, dass dieses Element der gerade geloeschten
 // Spalte die Nicht-Gewaehlt-Grundfarbe bekommen soll. Da dann dieses Element aber nicht mehr existiert,
 // kommt es zu einer Speicherverletzung und Abbruch.
 if (Element1Gewaehlt == this)
  {
   Element1Gewaehlt = NULL;
   Element2Gewaehlt = NULL;
  }
}  // ~ElementEventBoxClass


void ElementEventBoxClass::SetElementVerzeichnis(const std::string& VerzeichnisName)
{
 VerzeichnisAlt = VerzeichnisNeu;
 VerzeichnisNeu = VerzeichnisName;
} // SetVerzeichnis


void ElementEventBoxClass::ElementUeberschreibeVerzeichnisNeu(const std::string& VerzeichnisName)
{
 VerzeichnisNeu = VerzeichnisName;
}  // UeberschreibeVerzeichnisNeu


void ElementEventBoxClass::SetElementFilename(const std::string& FileName)
{
 FilenameAlt = FilenameNeu;
 FilenameNeu = FileName; 
 ElementLabelFilename.set_label(Glib::filename_to_utf8(FilenameNeu));
} // SetFilename


void ElementEventBoxClass::ElementUeberschreibeFilenameNeu(const std::string& FileName)
{
 FilenameNeu = FileName;
 ElementLabelFilename.set_label(Glib::filename_to_utf8(FilenameNeu)); 
} // UeberschreibeFilenameNeu 


void ElementEventBoxClass::SetElementBewertung(short Bewertung)
{
 BewertungAlt = BewertungNeu;
 BewertungNeu = Bewertung;
} // SetBewertung 


void ElementEventBoxClass::SetElementSichtbar(bool Sichtbarkeit)
{
 Sichtbar = Sichtbarkeit;
} // SetSichtbar 


void ElementEventBoxClass::SetElementFileGespeichert()
{
 FilenameAlt = FilenameNeu;
 VerzeichnisAlt = VerzeichnisNeu;
} // SetFileGespeichert 


void ElementEventBoxClass::SetElementBewertungGespeichert()
{
 BewertungAlt = BewertungNeu;
} // SetBewertungGespeichert 


void ElementEventBoxClass::SetElementDrehungGespeichert()
{
 Drehung = 0;
}  // SetDrehungGespeichert


void ElementEventBoxClass::SetElementUmbenannt(bool Umbenannt)
{
 ElementUmbenannt = Umbenannt;
}  // SetUmbenannt


void ElementEventBoxClass::SetElementUmbewertet(bool Umbewertet)
{
 ElementUmbewertet = Umbewertet;
}  // SetUmbewertet


const std::string& ElementEventBoxClass::GetElementVerzeichnisNeu()
{
 return VerzeichnisNeu;
} // GetVerzeichnisNeu 


const std::string& ElementEventBoxClass::GetElementVerzeichnisAlt()
{
 return VerzeichnisAlt;
} // GetVerzeichnisAlt 


const std::string& ElementEventBoxClass::GetElementFilenameNeu()
{
 return FilenameNeu;
} // GetFilenameNeu 


const std::string& ElementEventBoxClass::GetElementFilenameAlt()
{
 return FilenameAlt;
} // GetFilenameAlt 


const std::string& ElementEventBoxClass::GetElementDatum()
{
 return Datum;
} // GetDatum 


short ElementEventBoxClass::GetElementBewertungNeu()
{
 return BewertungNeu;
} // GetBewertungNeu 


short ElementEventBoxClass::GetElementBewertungAlt()
{
 return BewertungAlt;
} // GetBewertungAlt 


bool ElementEventBoxClass::GetElementSichtbar()
{
 return Sichtbar;
} // GetSichtbar 


bool ElementEventBoxClass::GetElementUmbenannt()
{
 return ElementUmbenannt;
}  // GetUmbenannt


bool ElementEventBoxClass::GetElementUmbewertet()
{
 return ElementUmbewertet;
}  // GetUmbewertet


int ElementEventBoxClass::GetElementDrehung()
{
 return Drehung;
}  // GetDrehung


bool ElementEventBoxClass::GetIstStartEndeElement()
{
 return StartEndeElement;
}  // GetIstStartEndeElement


bool ElementEventBoxClass::ElementVerzeichnisGeaendert()
{
 return (VerzeichnisAlt != VerzeichnisNeu);
} // VerzeichnisGeaendert 


bool ElementEventBoxClass::ElementFilenameGeaendert()
{
 return (FilenameAlt != FilenameNeu);
} // FilenameGeaendert 


bool ElementEventBoxClass::ElementBewertungGeaendert()
{
 return (BewertungAlt != BewertungNeu);
} // BewertungGeaendert 


bool ElementEventBoxClass::ElementDrehungGeaendert()
{
 return (Drehung != 0);
}  // DrehungGeaendert

ElementEventBoxClass* ElementEventBoxClass::GetElement1Gewaehlt()
{
 return Element1Gewaehlt;
}  // GetElement1Gewaehlt


ElementEventBoxClass* ElementEventBoxClass::GetElement2Gewaehlt()
{
 return Element2Gewaehlt;
}  // GetElement2Gewaehlt


void ElementEventBoxClass::ElementRechtsDrehen()
{
 if (Element1Gewaehlt)
  {
   Glib::RefPtr<Gdk::Pixbuf> BildBufferAlt(Element1Gewaehlt->ElementImage.get_pixbuf());
   Glib::RefPtr<Gdk::Pixbuf> BildBufferNeu(BildBufferAlt->rotate_simple(Gdk::PIXBUF_ROTATE_CLOCKWISE));
   Element1Gewaehlt->ElementImage.set(BildBufferNeu);
   Element1Gewaehlt->ElementImage.show();
   if (Element1Gewaehlt->Drehung < 270)
    {
     Element1Gewaehlt->Drehung += 90;
    }
   else
    {
     Element1Gewaehlt->Drehung = 0;
    }
  }
}  // ElementRechtsDrehen


void ElementEventBoxClass::ElementLinksDrehen()
{
 if (Element1Gewaehlt)
  {
   Glib::RefPtr<Gdk::Pixbuf> BildBufferAlt(Element1Gewaehlt->ElementImage.get_pixbuf());
   Glib::RefPtr<Gdk::Pixbuf> BildBufferNeu(BildBufferAlt->rotate_simple(Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE));
   Element1Gewaehlt->ElementImage.set(BildBufferNeu);
   Element1Gewaehlt->ElementImage.show();
   if (Element1Gewaehlt->Drehung > 0)
    {
     Element1Gewaehlt->Drehung -= 90;
    }
   else
    {
     Element1Gewaehlt->Drehung = 270;
    }
  }
}  // ElementLinksDrehen


void ElementEventBoxClass::ElementBetrachten()
{
 if (!Element1Gewaehlt)
  {
   return;
  }
 ElementBetrachtenWindowClassManager Betrachtungsfenster;
 Betrachtungsfenster.WindowOeffnen(*Element1Gewaehlt);
}  // ElementBetrachten


void ElementEventBoxClass::ElementEditieren()
{
 // Zeiger merken, da der originale ja veraendert werden koennte:
 ElementEventBoxClass* ElementZeiger = Element1Gewaehlt;
 if (!ElementZeiger)
  {
   return;
  }
 // Aufrufzeile fuer den gimp. Bild sollte von gimp unter dem alten Namen gespeichert werden:
 std::string VerzeichnisFilename(ElementZeiger->GetElementVerzeichnisAlt() + "/" +
                                 ElementZeiger->GetElementFilenameAlt());
 std::string StringBefehl("gimp " + VerzeichnisFilename);
 char* cBefehl = new char[static_cast<int>(StringBefehl.size()) + 1];
 strcpy(cBefehl, StringBefehl.c_str()); 
 int Fehler = system(cBefehl);
 // Wenn der Gimp auf dem System nicht installiert ist:
 if (Fehler)
  {
   Gtk::MessageDialog *SpeicherfehlerDialog =
    new Gtk::MessageDialog(_("System call failed!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
   SpeicherfehlerDialog->set_secondary_text
    (_("Cannot start gimp. Is gimp installed?"));
   SpeicherfehlerDialog->run();
   delete SpeicherfehlerDialog;
   delete[] cBefehl;
   return;
  }
 delete[] cBefehl;
 // Editiertes Bildchen neu laden:
 Glib::RefPtr<Gdk::Pixbuf> ElementPixbufQuelle;
 Glib::RefPtr<Gdk::Pixbuf> ElementPixbufZiel;
 short BildAusrichtung;
 Gdk::PixbufRotation GdKRotation;
 try
  {
   Exiv2::Image::AutoPtr JpegFile = Exiv2::ImageFactory::open(VerzeichnisFilename);
   JpegFile->readMetadata();
   // Exiv2-Daten in Pairs-Container einsortieren:
   Exiv2::ExifData &JpegExifData = JpegFile->exifData();
   if (JpegExifData.empty())
    {
     std::cerr << "ElementEditieren: Exif2-data unavailable." << std::endl;
     BildAusrichtung = 1;
    } 
   else
    {
     // Auf die interessanten Exiv-Daten im Container zugreifen:
     Exiv2::Exifdatum& JpegTag = JpegExifData["Exif.Image.Orientation"];
     BildAusrichtung = static_cast<short>(JpegTag.toLong());
    }
  }
 catch(...)
  {
   std::cerr << "ElementEditieren: Exception thrown by Exif2." << std::endl;   
   BildAusrichtung = 1;
  }
 switch (BildAusrichtung)
  {
   case 1:
    ElementZeiger->Drehung = 0;
    GdKRotation = Gdk::PIXBUF_ROTATE_NONE;
    break;
   case 3:
    ElementZeiger->Drehung = 180;
    GdKRotation = Gdk::PIXBUF_ROTATE_UPSIDEDOWN;
    break;
   case 6:
    ElementZeiger->Drehung = 90;
    GdKRotation = Gdk::PIXBUF_ROTATE_CLOCKWISE;
    break;
   case 8:
    ElementZeiger->Drehung = 270;
    GdKRotation = Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE;
    break;
   default:
    ElementZeiger->Drehung = 0;
    GdKRotation = Gdk::PIXBUF_ROTATE_NONE;
    break;
  }
 // Verkleinertes Bild aus dem File holen:   
 try
  {
   ElementPixbufQuelle = Gdk::Pixbuf::create_from_file(VerzeichnisFilename, 100, 100);
   ElementPixbufZiel = ElementPixbufQuelle->rotate_simple(GdKRotation);  
  }
 catch(...)
  {
   std::cerr << "FilehandlingElementLaden: Exception thrown by file-, I/O- or pixelbuffer-operation." << std::endl;
   ElementPixbufZiel = GenerellesPixelbufferSW;
  }
 ElementZeiger->ElementImage.set(ElementPixbufZiel);
 ElementZeiger->show();
}  // ElementEditieren


void ElementEventBoxClass::ElementBewertungsButtonHandler(int ButtonNummer)
{
 SetElementBewertung(ButtonNummer);
 ElementUmbewertet = true;
} // ElementBewertungsButtonHandler


bool ElementEventBoxClass::ElementEnterNotifyHandler(GdkEventCrossing* event)
{
 override_background_color(GenerellesElementFarbeAktiv);
 // Da Signalhandler nicht weitergereicht werden muss, true zurueckgeben:
 return true;
}  // ElementEnterNotifyHandler


bool ElementEventBoxClass::ElementLeaveNotifyHandler(GdkEventCrossing* ElementLeaveNotifyEvent)
{
 if (ElementGewaehlt)
  {
   override_background_color(GenerellesElementFarbeGewaehlt);
  }
 else 
  {
   override_background_color(GenerellesElementFarbeNormal);
  } 
 // Da Signalhandler nicht weitergereicht werden muss, true zurueckgeben:
 return true;
}  // ElemenLeaveNotifyHandler


bool ElementEventBoxClass::ElementButtonPressHandler(GdkEventButton* ElementButtonPressEvent)
{
 // Ganz zuerst testen, ob das ein Doppelklick ist. Doppelklick gilt bei 2 Klicks innerhalb von 250ms.
 // Zu beachten ist, dass der ElementButtonPressHandler  beim Doppelklick mehrfach aufgerufen wird:
 // GDK_BUTTON_PRESS => GDK_BUTTON_RELEASE => GDK_BUTTON_PRESS => GDK_2BUTTON_PRESS => GDK_BUTTON_RELEASE.
 if (ElementButtonPressEvent->type == GDK_2BUTTON_PRESS)
  {
   // Hier nur das Bild aufziehen:
   ElementBetrachten();  
   // Signallinie ist hiermit beendet, daher true zurueckgeben: 
   return true;
  }
 // Das war also kein Doppelklick, also liegt ein Drag-Drop-Aufruf vor. Dass bei einem Doppelklick
 // GDK_BUTTON_PRESS ein zweites Mal ausgeloest wird, macht nichts aus, denn die unten programmierten
 // Abfragen fangen das auf.
 // Dann also zuerst testen, dass das geklickte Element nicht das Abschlusselement der Spalte ist.
 // Wenn es das ist, dann nur deselektieren:
 if (StartEndeElement)
  {
   // Status eine Auswahl:
   if ((Element1Gewaehlt != NULL) && (Element2Gewaehlt == NULL))
    {
     Element1Gewaehlt->override_background_color(GenerellesElementFarbeNormal); 
     Element1Gewaehlt->ElementGewaehlt = false;
     Element1Gewaehlt = NULL;
     NrSpalteAktuellGewaehlt = 0;
    }
   // Status zwei Auswahlen:
   else
    {
     if ((Element1Gewaehlt != NULL) && (Element2Gewaehlt != NULL))
      {
       ElementEventBoxClass *Zeiger = Element1Gewaehlt;
       do
        {
         Zeiger->override_background_color(GenerellesElementFarbeNormal);
         Zeiger->ElementGewaehlt = false;
         Zeiger = Zeiger->ElementDanach;
         }
        while(Zeiger != Element2Gewaehlt->ElementDanach);
        Element1Gewaehlt = NULL;
        Element2Gewaehlt = NULL;
        NrSpalteAktuellGewaehlt = 0;
      }
    }
  }
 else
  {
   // Userabsicht ist erste Auswahl: 
   if (!(ElementButtonPressEvent->state & Gdk::SHIFT_MASK))
    {
     // Status keine Auswahl:   
     if ((Element1Gewaehlt == NULL) && (Element2Gewaehlt == NULL))
      {
       Element1Gewaehlt = this;
       ElementGewaehlt = true;
       override_background_color(GenerellesElementFarbeGewaehlt);
       NrSpalteAktuellGewaehlt = NrSpalteHier;     
      }
     else
      {
       // Status eine Auswahl:
       if (Element2Gewaehlt == NULL)
        {
         // ...und der Klick geht nicht ins schon gewaehlte Element:
         if (Element1Gewaehlt != this)
          {
           Element1Gewaehlt->override_background_color(GenerellesElementFarbeNormal); 
           override_background_color(GenerellesElementFarbeGewaehlt);
           Element1Gewaehlt->ElementGewaehlt = false;
           ElementGewaehlt = true;
           Element1Gewaehlt = this;
           NrSpalteAktuellGewaehlt = NrSpalteHier;
          }
        }
       // Status zwei Auswahlen:
       else
        {
         if (!ElementGewaehlt)
          {
           ElementEventBoxClass *Zeiger = Element1Gewaehlt;
           do
            {
             Zeiger->override_background_color(GenerellesElementFarbeNormal);
             Zeiger->ElementGewaehlt = false;
             Zeiger = Zeiger->ElementDanach;
            }
           while(Zeiger != Element2Gewaehlt->ElementDanach);
           Element2Gewaehlt = NULL;
           Element1Gewaehlt = this;
           override_background_color(GenerellesElementFarbeGewaehlt);
           ElementGewaehlt = true;
           NrSpalteAktuellGewaehlt = NrSpalteHier;
          }
        }
      }     
    }
   // Userabsicht ist zweite Auswahl:   
   else
    {
     // Status keine Auswahl:   
     if ((Element1Gewaehlt == NULL) && (Element2Gewaehlt == NULL))
      {
       override_background_color(GenerellesElementFarbeGewaehlt);
       Element1Gewaehlt = this;
       ElementGewaehlt = true;
       NrSpalteAktuellGewaehlt = NrSpalteHier;
      }
     else
      {
       // Status eine Auswahl:     
       if (Element2Gewaehlt == NULL)
        {
         // Ist es die selbe Spalte?
         if (NrSpalteAktuellGewaehlt == NrSpalteHier)
          {
           // Testen, ob Shift-Auswahl nach der Auswahl kommt
           ElementEventBoxClass *Zeiger = Element1Gewaehlt->ElementDanach;
           do
            {
             if (Zeiger == this)
              {
               break;
              }
             Zeiger = Zeiger->ElementDanach;
            }
           while (!Zeiger->StartEndeElement);
           // Wenn Shift-Auswahl nach der ersten Auswahl kommt:
           if (!Zeiger->StartEndeElement)
            {
             Element2Gewaehlt = this;
             Zeiger = Element1Gewaehlt;
             do
              {
               Zeiger = Zeiger->ElementDanach;
               Zeiger->ElementGewaehlt = true;
               Zeiger->override_background_color(GenerellesElementFarbeGewaehlt);
              }
             while(Zeiger != Element2Gewaehlt);
            }
           // Wenn Shift-Auswahl nicht nach der ersten Auswahl kommt, aber in der richtigen Spalte ist:
           else
            {
             Element1Gewaehlt->override_background_color(GenerellesElementFarbeNormal); 
             override_background_color(GenerellesElementFarbeGewaehlt);
             Element1Gewaehlt->ElementGewaehlt = false;
             ElementGewaehlt = true;
             Element1Gewaehlt = this;
            }      
          }
         // Es ist nicht die selbe Spalte:
         else
          {
           Element1Gewaehlt->override_background_color(GenerellesElementFarbeNormal); 
           override_background_color(GenerellesElementFarbeGewaehlt);
           Element1Gewaehlt->ElementGewaehlt = false;
           ElementGewaehlt = true;
           Element1Gewaehlt = this;
           NrSpalteAktuellGewaehlt = NrSpalteHier; 
          }
        }
       // Status zwei Auswahlen:
       else
        {
         if (!ElementGewaehlt)
          {
           ElementEventBoxClass *Zeiger = Element1Gewaehlt;
           do
            {
             Zeiger->override_background_color(GenerellesElementFarbeNormal);
             Zeiger->ElementGewaehlt = false;
             Zeiger = Zeiger->ElementDanach;
            }
           while(Zeiger != Element2Gewaehlt->ElementDanach);
           Element2Gewaehlt = NULL;
           Element1Gewaehlt = this;
           override_background_color(GenerellesElementFarbeGewaehlt);
           ElementGewaehlt = true;
           NrSpalteAktuellGewaehlt = NrSpalteHier;
          } 
        }
      }
    }   
  }  
 // Signalhandler muss an den Drag-Drop-Handler weitergereicht werden koennen, daher false zurueckgeben: 
 return false;
}  // ElementButtonPressHandler


bool ElementEventBoxClass::ElementDragDropHandler
 (const Glib::RefPtr<Gdk::DragContext>& Kontext, int X, int Y, guint Zeit)
{
 // Nur ein Element soll per Drag und Drop verschoben werden:
 if (Element2Gewaehlt == NULL)
  {
   // Man kann ja kein Element vor sich selbst einstellen:
   if (Element1Gewaehlt == this)
    {
     // Zurueckspringen und dabei Signalkette nicht unterbrechen- es koennte ja noch etwas kommen:
     return false;
    }
   // Wenn man ein Endeelement einer Spalte verschieben wollte, so ginge das ja auch nicht:
   if (Element1Gewaehlt == NULL)
    {
     // Auch hier: Zurueckspringen und dabei Signalkette nicht unterbrechen:
     return false;
    }
   // Zustandskontrollvariablen aktualisieren:
   if (Element1Gewaehlt->NrSpalteHier == NrSpalteHier)
    {
     GenerellesEditierspalteUmsortiert[NrSpalteHier] = true;    
    }
   else
    {    
     GenerellesEditierspalteHingeschoben[NrSpalteHier] = true;  
    }
   // Gewaehltes Element ist garantiert sichtbar, es wird daher in jedem Fall verschoben:
   // Gewaehltes Element aus dem Quell-Grid nehmen:
   Gtk::Container* ContainerZeiger = Element1Gewaehlt->get_parent();
   ContainerZeiger->remove(*Element1Gewaehlt);
   // Gewaehltes Element aus der Quellliste aushaengen:
   Element1Gewaehlt->ElementDavor->ElementDanach = Element1Gewaehlt->ElementDanach;
   Element1Gewaehlt->ElementDanach->ElementDavor = Element1Gewaehlt->ElementDavor;
   // Gewaehltes Element in die Zielliste einhaengen:
   Element1Gewaehlt->ElementDanach = this;
   Element1Gewaehlt->ElementDavor = this->ElementDavor;
   this->ElementDavor->ElementDanach = Element1Gewaehlt;
   this->ElementDavor = Element1Gewaehlt;
   // Gewaehltes Element mit normalem Hintergrund versehen:
   Element1Gewaehlt->override_background_color(GenerellesElementFarbeNormal);
   // Gewaehltes Element als nicht gewaehlt kennzeichnen:
   Element1Gewaehlt->ElementGewaehlt = false;
   // Gewaehltes Element in das Ziel-Grid einsetzen:
   ContainerZeiger = get_parent();
   Gtk::Grid* GridZeiger = static_cast<Gtk::Grid*>(ContainerZeiger);    
   GridZeiger->insert_next_to(*this, Gtk::POS_TOP);
   GridZeiger->attach_next_to(*Element1Gewaehlt, *this, Gtk::POS_TOP, 1, 1);
   // Gewaehltes Element: Daten aktualisieren:
   Element1Gewaehlt->NrSpalteHier = NrSpalteHier;
   Element1Gewaehlt->ElementUeberschreibeVerzeichnisNeu(GetElementVerzeichnisNeu());
   // Element1 deselektieren:
   Element1Gewaehlt = NULL;
  }
 // Gleich mehrere Elemente sollen verschoben werden, d.h. Element2Gewaehlt != NULL:
 else
  {
   // Zustandskontrollvariablen ueberpruefen:
   // ... Quelle und Ziel in der selben Spalte:
   if (Element1Gewaehlt->NrSpalteHier == NrSpalteHier)
    {
     // Dann also testen, ob es versucht wird, zu einem ausgewaehlten Element hin zu verschieben,
     // was ja nicht gehen kann. Hier werden alle Elemente zwischen Element1Gewaehlt und Element2Gewaewhlt
     // intersucht, egal ob sichtbar oder unsichtbar, weil die ganze Kette mit allen sichtbaren
     // und unsichtbaren Elementen die Obermenge darstellt und wegen der Verkettungsstruktur auch durch-
     // laufen werden muss:
     for (ElementEventBoxClass* ElementZeiger = Element1Gewaehlt;
          ElementZeiger != Element2Gewaehlt->ElementDanach;
          ElementZeiger = ElementZeiger->ElementDanach)
      {
       if (ElementZeiger == this)
        {
         return false;
        }
      }
     // Es wird also innerhalb einer Spalte umsortiert:
     GenerellesEditierspalteUmsortiert[NrSpalteHier] = true;
    }
   // ... Quelle und Ziel sind nicht in der selben Spalte:
   // Auch hier muss die Sichtbarkeit nicht geprueft werden, weil Element1Gewaehlt und Element2Gewaehlt
   // sicher sichtbar sind und sicher verschoben werden:
   else
    {      
     GenerellesEditierspalteHingeschoben[NrSpalteHier] = true;  
    }      
   // Quell- und Ziel-Container-Grids der grafischen Elemente ermitteln:
   Gtk::Container* ContainerZeigerQuelle = Element1Gewaehlt->get_parent();
   Gtk::Grid* GridZeigerQuelle = static_cast<Gtk::Grid*>(ContainerZeigerQuelle);
   Gtk::Container* ContainerZeigerZiel = get_parent();
   Gtk::Grid* GridZeigerZiel = static_cast<Gtk::Grid*>(ContainerZeigerZiel);
   // Gewaehlte Elemente aus dem Quell-Grid nehmen und im Zielgrid einsetzen,
   // dabei in den Elementen Verzeichnis, Hintergrund und Gewaehlt-Flag aktualisieren:
   ElementEventBoxClass* Zeiger;  
   // Nur damit man in einem Rutsch durchiterieren kann:
   Element2Gewaehlt = Element2Gewaehlt->ElementDanach;
   do
    {     
     // QuellenZeiger darf nicht Manipuliert werden. Zum Zeitpunkt der Adressmanipulationen
     // muss der QuellenZeiger bereits in Sicherheit gebracht sein:
     Zeiger = Element1Gewaehlt;
     Element1Gewaehlt = Element1Gewaehlt->ElementDanach;
     // Nur die Elemente zwischen Element1Gewaehlt und Element2Gewaehlt umhaengen, die sichtbar sind:  
     if (Zeiger->Sichtbar)
      {
       // Zeiger aus der Quellliste aushaengen:       
       Zeiger->ElementDavor->ElementDanach = Element1Gewaehlt;
       Element1Gewaehlt->ElementDavor = Zeiger->ElementDavor;
       // Gewaehltes, ausgehaengtes Element in die Zielliste einhaengen:
       Zeiger->ElementDavor = this->ElementDavor;
       Zeiger->ElementDanach = this;
       this->ElementDavor->ElementDanach = Zeiger;
       this->ElementDavor = Zeiger;    
       // Die Grafikelemente verschieben, dabei die Kontrollvariablen aktualisieren:
       GridZeigerQuelle->remove(*Zeiger);
       GridZeigerZiel->insert_next_to(*this, Gtk::POS_TOP);
       Zeiger->NrSpalteHier = NrSpalteHier;       
       Zeiger->ElementUeberschreibeVerzeichnisNeu(GetElementVerzeichnisNeu());
       Zeiger->override_background_color(GenerellesElementFarbeNormal);
       Zeiger->ElementGewaehlt = false;
       GridZeigerZiel->attach_next_to(*Zeiger, *this, Gtk::POS_TOP, 1, 1);
      }
    }
   while(Element1Gewaehlt != Element2Gewaehlt); 
   // Element1 und Element2 deselektieren:
   Element1Gewaehlt = NULL;
   Element2Gewaehlt = NULL;
  }
 // Da Signalhandler ggf. weitergereicht werden muss, false zurueckgeben. Das Schlimmste, das im
 // Moment so passieren kann, ist doch, dass ein gtkmm-Stup die Signalkette abschliesst: 
 return false;
}  // ElementDragDropHandler


//************************************************************************************************************


ElementBetrachtenWindowClass::ElementBetrachtenWindowClass(ElementEventBoxClass& Bild)
 : RahmenstaerkeHorizontal(20), RahmenstaerkeVertikal(45)
{
 // Adresse des Bildes zuest einmal sichern fuer das Weiterblaettern:
 ZuZeigendesBild = &Bild;
 // Verzeichnis und Filename, sowie Drehung des Bildes verfuegbar machen:
 std::string VerzeichnisFileName =
  ZuZeigendesBild->GetElementVerzeichnisAlt() + "/" + ZuZeigendesBild->GetElementFilenameAlt();
 int Drehung = ZuZeigendesBild->GetElementDrehung();
 // Callback fuer das Aufziehen einhaengen. Ich habe keine andere als die allgemeine Signalfunktion zum
 // Laufen gebracht:
 signal_event().connect(sigc::mem_fun(*this, &ElementBetrachtenWindowClass::ElementBetrachtenKonfigureEvent));
 // Callback fuer das Schliessen einhaengen. Dies deshalb, weil das Element sich selbst deallokieren muss:
 signal_delete_event().connect(sigc::mem_fun(*this,&ElementBetrachtenWindowClass::ElementBetrachtenSchliessenEvent));
 // Callbacks fuer die Buttonsteuerung:
 ElementBetrachtenButtonVor.signal_clicked().connect
  (sigc::mem_fun(*this, &ElementBetrachtenWindowClass::ElementBetrachtenButtonVorClick));
 ElementBetrachtenButtonZurueck.signal_clicked().connect
  (sigc::mem_fun(*this, &ElementBetrachtenWindowClass::ElementBetrachtenButtonZurueckClick)); 
 // Grafik des neuen und extra Windows einstellen:
 set_title("PickSort");
 set_position(Gtk::WIN_POS_CENTER);
 set_keep_above(); 
 // Die zweite Ueberschrift zeigt Verzeichnis und Filename:
 ElementBetrachtenLabel.set_text(VerzeichnisFileName); 
 ElementBetrachtenLabel.set_ellipsize(Pango::ELLIPSIZE_START);
 ElementBetrachtenLabel.set_justify(Gtk::JUSTIFY_CENTER);
 ElementBetrachtenLabel.set_hexpand(true);
 // Zwei Buttons fuer das Vor- und Zurueckschalten:
 ElementBetrachtenButtonVor.set_label(">");
 ElementBetrachtenButtonVor.set_hexpand(false);
 ElementBetrachtenButtonVor.set_vexpand(false);
 ElementBetrachtenButtonZurueck.set_label("<");
 ElementBetrachtenButtonZurueck.set_hexpand(false);
 ElementBetrachtenButtonZurueck.set_vexpand(false); 
 // Bild aus dem File laden oder einen einfachen Ersatz vorhalten:
 try
  {
   ElementBetrachtenPixbufQuelle = Gdk::Pixbuf::create_from_file(VerzeichnisFileName);
  }
 catch(...)
  {
   Gtk::MessageDialog *SpeicherfehlerDialog =
    new Gtk::MessageDialog(_("Picture viewer reports a problem!"),
                              false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
   SpeicherfehlerDialog->set_secondary_text(_("Root causes are file- or pixelbuffer-operations."));
   SpeicherfehlerDialog->run();
   delete SpeicherfehlerDialog;   
   ElementBetrachtenPixbufQuelle = GenerellesPixelbufferSW;
   Drehung = 0;
  }
 switch (Drehung)
  {
   case 0:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_NONE);
    break;
   case 90:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_CLOCKWISE);
    break;
   case 180:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_UPSIDEDOWN);
    break;
   case 270:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE);
    break;   
  }
 // Dafuer sorgen, dass beim Aufziehen auch nichts verzerrt wird:
 AspektQuelle = ((float)ElementBetrachtenPixbufZwischen->get_height()) /
                ((float)ElementBetrachtenPixbufZwischen->get_width());
 int BreiteBild;
 int HoeheBild;
 // Landscape: 
 if (AspektQuelle <= 1.0)
  {
   BreiteBild = 500;
   HoeheBild = (int)((float)BreiteBild * AspektQuelle);
  }
 // Portrait:
 else
  {
   HoeheBild = 500;
   BreiteBild = (int)((float)HoeheBild / AspektQuelle);
  }
 ElementBetrachtenPixbufZiel
  = ElementBetrachtenPixbufZwischen->scale_simple(BreiteBild, HoeheBild, Gdk::INTERP_BILINEAR);
 ElementBetrachtenImage.set(ElementBetrachtenPixbufZiel);
 ElementBetrachtenImage.set_hexpand(true);
 ElementBetrachtenImage.set_vexpand(true);
 ElementBetrachtenImage.set_alignment(Gtk::ALIGN_CENTER, Gtk::ALIGN_CENTER);
 ElementBetrachtenButtonVor.set_margin_right(1);
 ElementBetrachtenButtonZurueck.set_margin_left(1);  
 ElementBetrachtenGrid.attach(ElementBetrachtenImage, 1, 1, 3, 1); 
 ElementBetrachtenGrid.attach(ElementBetrachtenButtonZurueck, 1, 2, 1, 1); 
 ElementBetrachtenGrid.attach(ElementBetrachtenLabel, 2, 2, 1, 1); 
 ElementBetrachtenGrid.attach(ElementBetrachtenButtonVor, 3, 2, 1, 1);

 add(ElementBetrachtenGrid);
 // Jetzt noch das Window aufziehen mit einem schmalen Rahmen um das Bild. Der Rahmen ist dann der
 // Spaltabstand, den die Aufziehroutine für das Verkleinern braucht:
 HoeheLabel = ElementBetrachtenButtonZurueck.get_height();
 BreiteWindowIst = BreiteBild + RahmenstaerkeHorizontal;
 // Faktor 1.5 hat sich so ergeben:
 HoeheWindowIst = HoeheBild + HoeheLabel + RahmenstaerkeVertikal;
 resize(BreiteWindowIst, HoeheWindowIst);
 show_all();
}  // ElementBetrachtenWindowClass


ElementBetrachtenWindowClass::~ElementBetrachtenWindowClass()
{
}  // ~ElementBetrachtenWindowClass


bool ElementBetrachtenWindowClass::ElementBetrachtenSchliessenEvent(GdkEventAny* Event)
{
 // Fenster schliessen:
 destroy_();
 // Allokierte Fensterstruktur wieder freigeben:
 delete this;
 // Alles gemacht, daher den Standard-Handler nicht mehr anhaengen:
 return true;
}  // ElementBetrachtenSchliessenEvent


bool ElementBetrachtenWindowClass::ElementBetrachtenKonfigureEvent(GdkEvent* Event)
{
 int BreiteWindowSoll, HoeheWindowSoll;
 int BreiteWindowDelta, HoeheWindowDelta;
 int BreiteWindowBild, HoeheWindowBild;
 float AspektWindowSoll;
 // Aktuelle Window-Masse holen:
 get_size(BreiteWindowSoll, HoeheWindowSoll);
 // Sollte das Fenster zu klein werden, dann mache einfach nichts mehr, dann wird das Gtk::Image nicht mehr
 // kleiner und die ScrolledWindow-Scrollbalken werden sichtbar:
 if ((BreiteWindowSoll < 100) || (HoeheWindowSoll < 100))
  {
   // Direkt weitergeben an den naechsten Handler:
   return false;
  } 
 // Eine Lose einfuehren gegen allzu lahmes verkleinerndes Aktualisieren des Fensters:
 BreiteWindowDelta = BreiteWindowSoll - BreiteWindowIst;
 HoeheWindowDelta = HoeheWindowSoll - HoeheWindowIst;
 if ((BreiteWindowDelta > 1) || (BreiteWindowDelta < 3) || (HoeheWindowDelta > 1) || (HoeheWindowDelta < 3))
  {
   // AkspektVerhaeltnis des angeforderten Fensters berechnen
   AspektWindowSoll = (float)(HoeheWindowSoll - HoeheLabel - RahmenstaerkeVertikal) /
                      (float)(BreiteWindowSoll - RahmenstaerkeHorizontal);
   if (AspektWindowSoll >= AspektQuelle)
    {
     BreiteWindowBild = BreiteWindowSoll - RahmenstaerkeHorizontal;
     HoeheWindowBild = (int)((float)BreiteWindowBild * AspektQuelle);
    }
   else
    {
     HoeheWindowBild = HoeheWindowSoll - HoeheLabel - RahmenstaerkeVertikal;
     BreiteWindowBild = (int)((float)HoeheWindowBild / AspektQuelle);
    }                            
   ElementBetrachtenPixbufZiel =
    ElementBetrachtenPixbufZwischen->scale_simple(BreiteWindowBild, HoeheWindowBild, Gdk::INTERP_BILINEAR);
   ElementBetrachtenImage.set(ElementBetrachtenPixbufZiel);     
   // Aus Neu mach Alt:
   BreiteWindowIst = BreiteWindowSoll;
   HoeheWindowIst = HoeheWindowSoll;
  }
 // Die weiteren moeglichen Handler auch noch anspringen lassen: 
 return false;
}  // ElementBetrachtenKonfigureEvent


void ElementBetrachtenWindowClass::ElementBetrachtenButtonVorClick()
{
 // Nächtes Element anspringen, solange es nicht das Endeelement ist:
 ElementEventBoxClass* NaechstesElement = ZuZeigendesBild->ElementDanach;
 if (NaechstesElement->GetIstStartEndeElement())
  {
   return;
  }
 else
  {
   ZuZeigendesBild = NaechstesElement;
   ElementBetrachtenVorZurueckBlaettern();
  }
}  //  ElementBetrachtenButtonVorClick


void ElementBetrachtenWindowClass::ElementBetrachtenButtonZurueckClick()
{
 // Nächtes Element anspringen, solange es nicht das Endeelement ist:
 ElementEventBoxClass* VorherigesElement = ZuZeigendesBild->ElementDavor;
 if (VorherigesElement->GetIstStartEndeElement())
  {
   return;
  }
 else
  {
   ZuZeigendesBild = VorherigesElement;
   ElementBetrachtenVorZurueckBlaettern();
  }
}  // ElementBetrachtenButtonZurueckClick


void ElementBetrachtenWindowClass::ElementBetrachtenVorZurueckBlaettern()
{
 // Alte Speicherbelegungen verwerfen: 
 ElementBetrachtenPixbufQuelle.clear();
 ElementBetrachtenPixbufZwischen.clear(); 
 ElementBetrachtenPixbufZiel.clear();
 // Verzeichnis und Filename, sowie Drehung des Bildes verfuegbar machen:
 std::string VerzeichnisFileName =
  ZuZeigendesBild->GetElementVerzeichnisAlt() + "/" + ZuZeigendesBild->GetElementFilenameAlt();
 int Drehung = ZuZeigendesBild->GetElementDrehung(); 
 // Neues Bild laden und auf den Rahmen einpassen:
 // Dazu Fenstermasse holen und berechnen:
 int BreiteWindowIst, HoeheWindowIst; 
 get_size(BreiteWindowIst, HoeheWindowIst); 
 int BreiteWindowBild = BreiteWindowIst - RahmenstaerkeHorizontal; 
 int HoeheWindowBild = HoeheWindowIst - HoeheLabel - RahmenstaerkeVertikal;
 float AspektBildRahmen = (float)HoeheWindowBild / (float)BreiteWindowBild;
 // Dazu Bild laden und Bildmasse berechnen:
 try
  {
   ElementBetrachtenPixbufQuelle = Gdk::Pixbuf::create_from_file(VerzeichnisFileName);
  }
 catch(...)
  {
   Gtk::MessageDialog *SpeicherfehlerDialog =
    new Gtk::MessageDialog(_("Picture viewer reports a problem!"),
                              false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
   SpeicherfehlerDialog->set_secondary_text(_("Root causes are file- or pixelbuffer-operations."));
   SpeicherfehlerDialog->run();
   delete SpeicherfehlerDialog;   
   ElementBetrachtenPixbufQuelle = GenerellesPixelbufferSW;
   Drehung = 0;
  }
 switch (Drehung)
  {
   case 0:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_NONE);
    break;
   case 90:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_CLOCKWISE);
    break;
   case 180:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_UPSIDEDOWN);
    break;
   case 270:
    ElementBetrachtenPixbufZwischen
     = ElementBetrachtenPixbufQuelle->rotate_simple(Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE);
    break;   
  }
 AspektQuelle = ((float)ElementBetrachtenPixbufZwischen->get_height()) /
                ((float)ElementBetrachtenPixbufZwischen->get_width());
 if (AspektBildRahmen >= AspektQuelle)  // Rahmen 'portraitiger' das Bild
  { 
   HoeheWindowBild = (int)((float)BreiteWindowBild * AspektQuelle);
   ElementBetrachtenPixbufZiel =
    ElementBetrachtenPixbufZwischen->scale_simple(BreiteWindowBild, HoeheWindowBild, Gdk::INTERP_BILINEAR); 
   ElementBetrachtenImage.clear();
   ElementBetrachtenLabel.set_text(VerzeichnisFileName);   
   ElementBetrachtenImage.set(ElementBetrachtenPixbufZiel);  
  }
 else  // Rahmen 'landscapeiger' als das Bild
  {
   BreiteWindowBild = (int)((float)HoeheWindowBild / AspektQuelle);
   ElementBetrachtenPixbufZiel =
    ElementBetrachtenPixbufZwischen->scale_simple(BreiteWindowBild, HoeheWindowBild, Gdk::INTERP_BILINEAR);
   ElementBetrachtenImage.clear();
   ElementBetrachtenLabel.set_text(VerzeichnisFileName);
   ElementBetrachtenImage.set(ElementBetrachtenPixbufZiel);
  }
}  // ElementBetrachtenVorZurueckBlaettern


//************************************************************************************************************


ElementBetrachtenWindowClassManager::ElementBetrachtenWindowClassManager()
{
}  // ElementBetrachtenWindowClassManager


ElementBetrachtenWindowClassManager::~ElementBetrachtenWindowClassManager()
{
}  // ~ElementBetrachtenWindowClassManager


void ElementBetrachtenWindowClassManager::WindowOeffnen(ElementEventBoxClass& ZuZeigendesBild)
{
 // Dummy wird spaeter nicht gebraucht, weil sich das Fenster selbst deallokiert.
 // Das Fenster wird also hier allokiert und es bleibt nach Ende der Funktion sich selbst ueberlassen:
 ElementBetrachtenWindowClass* Dummy = new ElementBetrachtenWindowClass(ZuZeigendesBild);
}  // BetrachtungsfensterOeffnen


/*
 * PickSortElement.cpp ***************************************************************************************
 */

