/*
 *************************************************************************************************************
 * PickSortFilehandling.cpp     
 *
 * Alle File- und Verzeichnisinteraktionen werden hier gekapselt.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortFilehandling.h"


int FilehandlingClass::FilehandlingFilesAuswaehlen()
{
 Gtk::FileChooserDialog VerzeichnisDialog
  (_("Choose JPEGs in source directory"), Gtk::FILE_CHOOSER_ACTION_OPEN);
 VerzeichnisDialog.set_select_multiple();
 Glib::RefPtr<Gtk::FileFilter> VerzeichnisFilter = Gtk::FileFilter::create();
 VerzeichnisFilter->add_mime_type("image/jpeg"); 
 VerzeichnisDialog.set_filter(VerzeichnisFilter);
 VerzeichnisDialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
 VerzeichnisDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 if (VerzeichnisDialog.run() == Gtk::RESPONSE_OK)
  {
   VerzeichnisName = VerzeichnisDialog.get_current_folder();
   if (VerzeichnisName.empty())
    {
     Gtk::MessageDialog *KeinVerzeichnisDialog =
      new Gtk::MessageDialog
       (_("Directory not found!"), false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK, true);
     KeinVerzeichnisDialog->set_secondary_text
      (_("Pick JPEGs only from real directory entries."));
     KeinVerzeichnisDialog->run();
     delete KeinVerzeichnisDialog;
     return 0;
    }  
   FileNamensVector = VerzeichnisDialog.get_filenames();
   if (FileNamensVector.empty())
    {
     return 0;
    }
   else
    {
     FilenamensIterator = FileNamensVector.begin();
     return (int)FileNamensVector.size();     
    }
  }
 else
  {
   return 0;
  }
}  // FilehandlingFilesAuswaehlen


const std::string& FilehandlingClass::GetFilehandlingVerzeichnisName()
{
 return VerzeichnisName;
}  // GetFilehandlingVerzeichnisName 


ElementEventBoxClass* FilehandlingClass::FilehandlingElementLaden(unsigned short SpaltenNummer,
                                                                  unsigned short BufferPixelAnzahl)
{
 ElementEventBoxClass* ElementHandler;
 Glib::RefPtr<Gdk::Pixbuf> ElementPixbufQuelle;
 Glib::RefPtr<Gdk::Pixbuf> ElementPixbufZiel; 
 std::string JpegAufnahmezeitpunkt;
 short JpegBewertung;
 short BildAusrichtung;
 int BildDrehung;
 Gdk::PixbufRotation GdKRotation;
 // Filename und Pfad aus der absoluten Pfadangabe extrahieren:
 std::string::size_type Position = FilenamensIterator->find_last_of('/');
 std::string FileName(FilenamensIterator->begin() + Position + 1, FilenamensIterator->end());
 std::string FilePfad(FilenamensIterator->begin(), FilenamensIterator->begin() + Position);   
 // Der Laufzeit-Overhead der try-catch-Technik ist gross. Daher sollte man die Schleife in den try-Block
 // legen. Da ich hier aber komplexe Fileoperationen mache, fällt der genannte Overhead nicht mehr ins
 // Gewicht und die catch-Funktionen können dort angeordnet sein, wo sie inhaltlich hingehoeren. 
 try
  {
   Exiv2::Image::AutoPtr JpegFile = Exiv2::ImageFactory::open(*FilenamensIterator);
   JpegFile->readMetadata();
   // Exiv2-Daten in Pairs-Container einsortieren:
   Exiv2::ExifData &JpegExifData = JpegFile->exifData();
   if (JpegExifData.empty())
    {
     std::cerr << "FilehandlingElementLaden: Exif2-data unavailable." << std::endl;
     JpegAufnahmezeitpunkt = "?";
    } 
   else
    {
     // Auf die interessanten Exiv-Daten im Container zugreifen:
     Exiv2::Exifdatum& JpegTag = JpegExifData["Exif.Photo.DateTimeOriginal"];  
     JpegAufnahmezeitpunkt = JpegTag.toString();
     JpegTag = JpegExifData["Exif.Image.Orientation"];
     BildAusrichtung = static_cast<short>(JpegTag.toLong());
    }
   // Xmp-daten in Pairs-Container einsortieren:
   Exiv2::XmpData &JpegXmpData = JpegFile->xmpData();
   if (JpegXmpData.empty())
    {
     std::cerr << "FilehandlingElementLaden: Xmp-data unavailable." << std::endl;
     JpegBewertung = 0;
    }
   else
    {
     // Auf die interessanten Xmp-Daten im Container zugreifen:    
     Exiv2::Xmpdatum& XmpTag = JpegXmpData["Xmp.xmp.Rating"];
     JpegBewertung = XmpTag.toLong();
    }
  }
 catch(...)
  {
   std::cerr << "FilehandlingElementLaden: Exception thrown by Exif2." << std::endl;   
   JpegAufnahmezeitpunkt = "?";   
   JpegBewertung = 0;
  }
 switch (BildAusrichtung)
  {
   case 1:
    BildDrehung = 0;
    GdKRotation = Gdk::PIXBUF_ROTATE_NONE;
    break;
   case 3:
    BildDrehung = 180;
    GdKRotation = Gdk::PIXBUF_ROTATE_UPSIDEDOWN;
    break;
   case 6:
    BildDrehung = 90;
    GdKRotation = Gdk::PIXBUF_ROTATE_CLOCKWISE;
    break;
   case 8:
    BildDrehung = 270;
    GdKRotation = Gdk::PIXBUF_ROTATE_COUNTERCLOCKWISE;
    break;
   default:
    BildDrehung = 0;
    GdKRotation = Gdk::PIXBUF_ROTATE_NONE;
    break;
  }
 // Verkleinertes Bild aus dem File holen:   
 try
  {
   ElementPixbufQuelle = Gdk::Pixbuf::create_from_file(*FilenamensIterator, BufferPixelAnzahl, BufferPixelAnzahl);
   ElementPixbufZiel = ElementPixbufQuelle->rotate_simple(GdKRotation);  
  }
 catch(...)
  {
   std::cerr << "FilehandlingElementLaden: Exception thrown by file-, I/O- or pixelbuffer-operation." << std::endl;
   ElementPixbufZiel = GenerellesPixelbufferSW;
  }
 // ArbeitslistenVerkettungs-Element wird hier allokiert. Es wird beim Aufruf des Destruktors der zuge-
 // hoerigen Arbeitsliste wieder deallokiert.
 ElementHandler = new ElementEventBoxClass(FilePfad, FileName, JpegAufnahmezeitpunkt,
                                           JpegBewertung, true, ElementPixbufZiel,
                                           SpaltenNummer, BildDrehung);
 // Wenn die einlesende for-Schleife wirklich auf .size dimensioniert ist, dann steht der Iterator am Ende
 // auf .end. Wenn .end ausgelesen würde, dann ergäbe dies einen undefinierten Fehler, denn eine Feld-
 // grenzenkontrolle findet aus Performancegründen in den Containern nicht statt:
 ++FilenamensIterator; 
 return ElementHandler;
}  // FilehandlingElementLaden 


std::string FilehandlingClass::FilehandlingVerzeichnisAuswaehlen()
{
 std::string VerzeichnisName;
 
 Gtk::FileChooserDialog VerzeichnisDialog 
  (_("Create directory"), Gtk::FILE_CHOOSER_ACTION_CREATE_FOLDER);
 VerzeichnisDialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
 VerzeichnisDialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
 if (VerzeichnisDialog.run() == Gtk::RESPONSE_OK)
  {
   VerzeichnisName = VerzeichnisDialog.get_filename();
   if (VerzeichnisName.empty())
    {
     throw AusnahmeClass(AusnahmeClass::FILEHANDLINGKEINNAME,
                         "FilehandlingVerzeichnisAuswaehlen: Identifier string is empty.");
    }
   else
    {
     return VerzeichnisName;
    }
  }
 else
  {
   throw AusnahmeClass(AusnahmeClass::FILEHANDLINGEXIT,
                       "FilehandlingVerzeichnisAuswaehlen: User cancelled execution.");
  }
}  // FilehandlingVerzeichnisAuswaehlen


bool FilehandlingClass::FilehandlingElementSpeichern(ElementEventBoxClass* Element)
{
 // Zuerst eine geaenderte Bewertung oder Drehung in die Exif-Datensaetze des unverschobenen Files eintragen:
 // 0, 90, 180, 270 Grad Drehung durch den Benutzer angefordert:
 int BildDrehung = Element->GetElementDrehung(); 
 bool WertGeaendert = Element->ElementBewertungGeaendert();
 bool FileGeaendert = Element->ElementVerzeichnisGeaendert() or Element->ElementFilenameGeaendert();
 // Wenn nichts zu machen ist, dann schnell zurueck:
 if ((BildDrehung == 0) and !WertGeaendert and !FileGeaendert)
  {
   return true;
  }
 // Jetzt ist also etwas zu tun, daher
 // Quellfile-Name mit Pfad zusammensetzen:
 std::string FileNameQuelle = Element->GetElementVerzeichnisAlt() + "/" + Element->GetElementFilenameAlt(); 
 // Dann zuerst umfangreichere Aenderung der Bilddrehung verarbeiten:
 if (BildDrehung != 0)
  {
   // Zuerst die Exif-Daten aus dem alten File holen:
   // Datenhandler:
   Exiv2::XmpData JpegXmpData;
   Exiv2::ExifData JpegExifData;    
   try
    {
     // Filehandler:
     Exiv2::Image::AutoPtr JpegFile = Exiv2::ImageFactory::open(FileNameQuelle);
     if (!JpegFile->good())
      {
       throw AusnahmeClass(AusnahmeClass::FILEHANDLINGKEINOEFFNEN,
                           "FilehandlingElementSpeichern: Unable to open file.");
      }
     JpegFile->readMetadata();
     // Exif-Daten aus dem File holen:
     JpegXmpData = JpegFile->xmpData();
     JpegExifData = JpegFile->exifData();
     // Ende Scope JpegFile: Es wird hier wieder geschlossen:
    }
   catch(const AusnahmeClass& ExUndHopp)
    {
     std::cerr << ExUndHopp.GetAusnahmeText() << std::endl;
     return false;
    }
   // File mit Magick++ oeffnen und Bild drehen:
   try
    {
     Magick::Image Bild;
     Bild.read(FileNameQuelle);
     Bild.rotate(static_cast<double>(BildDrehung));
     // Meine Erfahrung: .write loescht den Exif-Block im File,
     // wahrscheinlich wird das File komplett ueberschrieben:
     Bild.write(FileNameQuelle);
     // Beim Verlassen des Scopes wird File geschlossen:
    }
   catch (...)
    {
     std::cerr << "FilehandlingElementSpeichern: Magick++-operation failed." << std::endl;
     return false;
    }  
   // Exif-Daten des neuen Files zugaenglich machen und die aktualisierten Daten eintragen:
   try
    {
     // Filehandler:
     Exiv2::Image::AutoPtr JpegFile = Exiv2::ImageFactory::open(FileNameQuelle);
     if (!JpegFile->good())
      {
       throw AusnahmeClass(AusnahmeClass::FILEHANDLINGKEINOEFFNEN,
                           "FilehandlingElementSpeichern: Unable to open file.");
      }  
     // Aenderungen in die Strukturen eintragen:
     // Checken, was eigentlich gespeichert werden muss:
     // Das Xmp.xmp.Rating-Tag ist ein Integer:
     JpegXmpData["Xmp.xmp.Rating"] = static_cast<int>(Element->GetElementBewertungNeu());
     // Speicherprozess im Element kenntlich machen:
     Element->SetElementBewertungGespeichert(); 
     // Das Exif.Image.Orientation-Tag ist ein Short: 
     JpegExifData["Exif.Image.Orientation"] = static_cast<short>(1);  
     // Speicherprozess im Element kenntlich machen:
     Element->SetElementDrehungGespeichert();
     // Schreibe die Daten dann ins File zurueck:     
     JpegFile->setExifData(JpegExifData);
     JpegFile->setXmpData(JpegXmpData);
     JpegFile->writeMetadata();
     // Ende Scope JpegFile: Es wird hier wieder geschlossen:     
    }
   catch(const AusnahmeClass& ExUndHopp)
    {
     std::cerr << ExUndHopp.GetAusnahmeText() << std::endl;
     return false;
    }
  }
 else
  {
   if (WertGeaendert)
    {   
     try
      {
       // Filehandler:
       Exiv2::Image::AutoPtr JpegFile = Exiv2::ImageFactory::open(FileNameQuelle);
       if (!JpegFile->good())
        {
         throw AusnahmeClass(AusnahmeClass::FILEHANDLINGKEINOEFFNEN,
                            "FilehandlingElementSpeichern: Unable to open file.");
        }
       JpegFile->readMetadata();       
       Exiv2::XmpData JpegXmpData = JpegFile->xmpData();
       // Das Xmp.xmp.Rating-Tag ist ein Integer:
       JpegXmpData["Xmp.xmp.Rating"] = static_cast<int>(Element->GetElementBewertungNeu());
       // Speicherprozess im Element kenntlich machen:
       Element->SetElementBewertungGespeichert();
       // Schreibe die Daten dann ins File zurueck:
       JpegFile->setXmpData(JpegXmpData);
       JpegFile->writeMetadata();
       // File wird dann wohl hier durch den Exiv2-Destruktor geschlossen.
      }   
     catch(const AusnahmeClass& ExUndHopp)
      {
       std::cerr << ExUndHopp.GetAusnahmeText() << std::endl;
       return false;
      }    
    }
  }
 // Veraenderte File- und Pfadnahmen fuehren zu move-Operationen am File:
 if (FileGeaendert)
  {
   // Ziel- und ggf. Quell-Filename mit Pfad zusammensetzen:
   std::string FileNameZiel = Element->GetElementVerzeichnisNeu() + "/" + Element->GetElementFilenameNeu();
   try
    {
     // create_for_path verfehlt nie, gibt aber ggf. einen Handler zurueck, der nicht funktioniert:
     Glib::RefPtr<Gio::File> FileHandlerQuelle = Gio::File::create_for_path(FileNameQuelle);
     Glib::RefPtr<Gio::File> FileHandlerZiel = Gio::File::create_for_path(FileNameZiel);
     // move wirft einen Gio::Error mit Code Gio::EXISTS, wenn ein Überschreiben drohte:
     FileHandlerQuelle->move(FileHandlerZiel, Gio::FILE_COPY_NONE);
     // Wenn alles glatt geht und move keine Ausnahme wirft, dann:
     Element->SetElementFileGespeichert();       
     return true;
    }
   catch(const Gio::Error& Ex)
    {
     // Ausnahme EXISTS muss abgefangen und verarbeitet werden, die andern fuehren zum Abbruch:
     if (Ex.code() != Gio::Error::EXISTS)
      {
       // Kritische Fehler: File nicht gefunden, Verzeichnis nicht existent, Schreibschutz, ...
       std::cerr << "FilehandlingElementSpeichern: System call 'mv' failed." << std::endl;
       return false;
      }
     else   
      {
       std::cerr << "FilehandlingElementSpeichern: Postfix necessary." << std::endl;  
       // Filenamen mit einem Postfix versehen und speichern:     
       std::string FileNameZwischen = Element->GetElementFilenameNeu();
       std::string::size_type Position = FileNameZwischen.find_last_of('.');
       FileNameZwischen.insert(Position, "_");  
       // Da nur der Name des neuen Files geaendert wurde,
       // darf der alte Filename nicht veraendert werden:
       Element->ElementUeberschreibeFilenameNeu(FileNameZwischen);
      }
    }
   // Zweiten Versuch starten, das jetzt im Namen modifizierte File zu verschieben:
   // Den modifizierten Filenamen mit Pfad zusammensetzen:
   FileNameZiel = Element->GetElementVerzeichnisNeu() + "/" + Element->GetElementFilenameNeu();   
   try
    {
     Glib::RefPtr<Gio::File> FileHandlerQuelle = Gio::File::create_for_path(FileNameQuelle);
     Glib::RefPtr<Gio::File> FileHandlerZiel = Gio::File::create_for_path(FileNameZiel);
     // move wirft einen Gio::Error mit Code Gio::EXISTS, wenn ein Überschreiben drohte:
     FileHandlerQuelle->move(FileHandlerZiel, Gio::FILE_COPY_NONE);
     // Verzeichnis und Filename auch im Element vermerken (nach Speichern soll Neu gleich Alt sein):
     Element->SetElementFileGespeichert();       
     return true;
    }
   catch(const Gio::Error& Ex)
    {  
     // Jetzt kann es nur noch schief gehen. Daher Fehler verarbeiten:
     std::cerr << "FilehandlingElementSpeichern: Memory error." << std::endl;
     return false;
    }
  }
 else
  {
   return true;
  }
}  // FilehandlingElementSpeichern


/*
 * PickSortFilehandling.cpp **********************************************************************************
 */

 


