/*
 *************************************************************************************************************
 * PickSortFilehandling.h       
 *
 * Alle konkreten File- und Verzeichnisinteraktionen werden hier gekapselt.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#ifndef PICKSORTFILEHANDLING_H
#define PICKSORTFILEHANDLING_H


#include <iostream>
#include <list>
#include <string>

#include <gtkmm.h>
#include <giomm.h>
#include <exiv2/image.hpp>
#include <exiv2/exif.hpp>

#include <Magick++.h>

#include "PickSortElement.h"
#include "PickSortGenerelles.h"
#include "PickSortAusnahme.h"


class FilehandlingClass : public GenerellesClass
{
 public:
  // Standard-Konstruktoren und Standard-Zuweisung reichen hier aus.
  // Fileauswahlbox laesst zu ladende Files auswaehlen. Diese werden in eine Liste eingestellt.
  // Rueckgabewert: Anzahl der zu ladenden Files:
  int FilehandlingFilesAuswaehlen();
  // Gibt den Verzeichnispfad zurueck:
  const std::string& GetFilehandlingVerzeichnisName();
  // Laden des jeweils naechsten Fileinhalts in einen Elementcontainer.
  // BufferPixel beschreibt die Laenge der langen Seite des Bildes.
  // Rueckgabewert: Das allokierte und frisch beladene Element. Dieses muss an anderer Stelle
  // dann wieder deallokiert werden:
  ElementEventBoxClass* FilehandlingElementLaden(unsigned short SpaltenNummer,
                                                 unsigned short BufferPixelAnzahl = 100);
  // Verzeichnisauswahlbox laesst Verzeichnis bestimmen, in das Files gespeichert werden sollen.
  // Rueckgabewert: Der Verzeichnispfad.
  // Sollte kein Verzeichnis ausgewaehlt worden sein, so wird eine Ausnahme KEINVERZEICHNIS geworfen:
  std::string FilehandlingVerzeichnisAuswaehlen();
  // Ein Element speichern. Wesentlich sind dabei die Angaben, die im jeweiligen Arbeitslisten-Element stehen.
  // Wenn nicht gespeichert werden konnte, so wird eine Ausnahme SPEICHERNMISSLUNGEN geworfen:
  bool FilehandlingElementSpeichern(ElementEventBoxClass* Element);  
 private:
  std::string VerzeichnisName;
  std::vector<std::string> FileNamensVector;
  std::vector<std::string>::iterator FilenamensIterator;
};  // class FilehandlingClass


#endif  // PICKSORTFILEHANDLING_H


/*
 * PickSortFilehandling.h ************************************************************************************
 */
