/*
 *************************************************************************************************************
 * PickSortGenerelles.cpp      
 *
 * Allgemeine Vereinbarungen, Konstanten, Grafiken, usf.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortGenerelles.h"


// Farbdefinitionen fuer die elemente der benutzeroberflaeche. Man findet die Farbdefinitionen,
// wenn man im Internet nach der Datei 'rgb.txt' sucht:
const Gdk::RGBA GenerellesClass::GenerellesSpalteFarbeNormal("blue4");
const Gdk::RGBA GenerellesClass::GenerellesSpalteFarbeAktiv("blue2");
const Gdk::RGBA GenerellesClass::GenerellesSpalteFarbeGewaehlt("red3");
const Gdk::RGBA GenerellesClass::GenerellesSpalteSchrift("gray100");
const Gdk::RGBA GenerellesClass::GenerellesElementFarbeNormal("gray20");
const Gdk::RGBA GenerellesClass::GenerellesElementFarbeAktiv("gray25");   
const Gdk::RGBA GenerellesClass::GenerellesElementFarbeGewaehlt("gray30"); 
const Gdk::RGBA GenerellesClass::GenerellesElementSchrift("gray80");                                                        
// Halten die Pixelpuffer dauerhaft und werden nur einmal zum Programmstart geladen:
Glib::RefPtr<Gdk::Pixbuf> GenerellesClass::GenerellesPixelbufferSW;
Glib::RefPtr<Gdk::Pixbuf> GenerellesClass::GenerellesPixelbufferRB;
std::vector<Glib::RefPtr<Gdk::Pixbuf> > GenerellesClass::GenerellesPixelbuffersIcons;
// Kontrollvariable fuer die Spalten. Wenn eine Element zur Spalte geschoben wurde, dann muss die Spalte
// gespeichert werden, sonst wird die Elementesammlung der Spalte ernsthaft unsynchron zu den Files in den
// Verzeichnissen. Die Variablen werden von den Elementen gesetzt und von den Spalten verarbeitet: 
std::vector<bool> GenerellesClass::GenerellesEditierspalteHingeschoben;
std::vector<bool> GenerellesClass::GenerellesEditierspalteUmsortiert;
// Kontrollvariable, auf dass nur einmal initialisiert wird:
bool GenerellesClass::GenerellesInitialisiert = false;


GenerellesClass::GenerellesClass()
{
 if (GenerellesInitialisiert)
  {
   return;
  }
 else
  {
   GenerellesInitialisiert = true;
   // Briefmarken laden:
   try
    {
     GenerellesPixelbufferSW = Gdk::Pixbuf::create_from_file("./PickSortIconSW.tif", 100, 100);
     GenerellesPixelbufferRB = Gdk::Pixbuf::create_from_file("./PickSortIconRB.tif", 300, 300);
     GenerellesPixelbuffersIcons.push_back(Gdk::Pixbuf::create_from_file("./PickSortIconRB.tif", 128, 128));
     GenerellesPixelbuffersIcons.push_back(Gdk::Pixbuf::create_from_file("./PickSortIconRB.tif",  64,  64));
     GenerellesPixelbuffersIcons.push_back(Gdk::Pixbuf::create_from_file("./PickSortIconRB.tif",  48,  48)); 
     GenerellesPixelbuffersIcons.push_back(Gdk::Pixbuf::create_from_file("./PickSortIconRB.tif",  32,  32)); 
     GenerellesPixelbuffersIcons.push_back(Gdk::Pixbuf::create_from_file("./PickSortIconRB.tif",  16,  16));
    }
   catch (const Glib::FileError& ExUndHopp1)
    {
     std::cerr << "GenerellesClass: Unable to open icon files." << std::endl;
    }
   catch (const Gdk::PixbufError& ExUndHopp2)
    { 
     std::cerr << "GenerellesClass: Unable to load icon data into pixelbuffer." << std::endl;
    }
  }
}  // GenerellesClass


GenerellesClass::~GenerellesClass()
{
}  // ~GenerellesClass


//************************************************************************************************************


bool GenerellesInternationalisierungClass::InterInitialisiert = false;
GenerellesInternationalisierungClass::InterSpracheKuerzel
 GenerellesInternationalisierungClass::AktuelleSprache = GenerellesInternationalisierungClass::SpracheDE;
GenerellesInternationalisierungClass::Phrase*
 GenerellesInternationalisierungClass::PhrasenMap[GenerellesInternationalisierungClass::SpracheANZAHL][256];

const uint32_t GenerellesInternationalisierungClass::FNV1aStartwert = 0x811c9dc5;
const uint32_t GenerellesInternationalisierungClass::FNV1aPrimzahl = 0x01000193;


GenerellesInternationalisierungClass::GenerellesInternationalisierungClass()
{
 if (!InterInitialisiert)
  {
   for (int Spalte = static_cast<int>(SpracheEN) + 1; Spalte < static_cast<int>(SpracheANZAHL) ; ++Spalte)
    {
     for (register int Zeile = 255; Zeile; --Zeile)
      {
       PhrasenMap[Spalte][Zeile] = NULL;
      }
     PhrasenMap[Spalte][0] = NULL;
    }
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "At least one column has been changed.\nTerminating without saving discards this preliminary work.",
    "At least one column has been changed.\nTerminating without saving discards this preliminary work.");
   SetInterString(SpracheDE,
    "At least one column has been changed.\nTerminating without saving discards this preliminary work.",
    "Mindestens eine Spalte wurde editiert.\nBeenden ohne Speichern verwirft diese Vorarbeiten.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Cannot find user manual",
    "Cannot find user manual");
   SetInterString(SpracheDE,
    "Cannot find user manual",
    "Handbuch kann nicht gefunden werden.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Cannot start gimp. Is gimp installed?",
    "Cannot start gimp. Is gimp installed?");
   SetInterString(SpracheDE,
    "Cannot start gimp. Is gimp installed?",
    "Kann gimp nicht starten. Ist gimp installiert?");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Cannot start PickSortPro.",
    "Cannot start PickSortPro.");
   SetInterString(SpracheDE,
    "Cannot start PickSortPro.",
    "PickSortPro kann nicht gestartet werden.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Choose JPEGs in source directory",
    "Choose JPEGs in source directory");
   SetInterString(SpracheDE,
    "Choose JPEGs in source directory",
    "JPEGs im Quellverzeichnis picken");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Choose sorting criterion",
    "Choose sorting criterion");
   SetInterString(SpracheDE,
    "Choose sorting criterion",
    "Sortierkriterium festlegen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Column",
    "Column");
   SetInterString(SpracheDE,
    "Column",
    "Spalte");          
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Column has been resorted, files have been renamed or rerated.\nCommand discards this premliminary work.",
    "Column has been resorted, files have been renamed or rerated.\nCommand discards this premliminary work.");
   SetInterString(SpracheDE,
    "Column has been resorted, files have been renamed or rerated.\nCommand discards this premliminary work.",
    "Spalte wurde umsortiert, Files umbenannt oder neu bewertet.\nKommando verwirft diese Vorarbeiten.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Column has been resorted but files not renamed.\nSaving discards this premliminary work.",
    "Column has been resorted but files not renamed.\nSaving discards this premliminary work.");
   SetInterString(SpracheDE,
    "Column has been resorted but files not renamed.\nSaving discards this premliminary work.",
    "Spalte wurde umsortiert und Files nicht neu benannt.\nDas Speichern verwirft diese Vorarbeiten.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Column will be sorted in ascending order of criterion.",
    "Column will be sorted in ascending order of criterion.");
   SetInterString(SpracheDE,
    "Column will be sorted in ascending order of criterion.",
    "Kriterium wird in aufsteigender Reihenfolge sortiert.");            
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Create directory",
    "Create directory");
   SetInterString(SpracheDE,
    "Create directory",
    "Verzeichnis erstellen");            
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Date",
    "Date");
   SetInterString(SpracheDE,
    "Date",
    "Datum");            
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Define column filter",
    "Define column filter");
   SetInterString(SpracheDE,
    "Define column filter",
    "Spaltenfilter festlegen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Directory",
    "Directory");
   SetInterString(SpracheDE,
    "Directory",
    "Verzeichnis");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Directory not found!",
    "Directory not found!");
   SetInterString(SpracheDE,
    "Directory not found!",
    "Kein Verzeichnis gefunden!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Directoy has been loaded into other column already.\nUnsynchonicity is imminent.",
    "Directoy has been loaded into other column already.\nUnsynchonicity is imminent.");
   SetInterString(SpracheDE,
    "Directoy has been loaded into other column already.\nUnsynchonicity is imminent.",
    "Das Verzeichnis wurde bereits in eine andere Spalte geladen.\nEs droht ein unsynchroner Zustand.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "edit",
    "edit");
   SetInterString(SpracheDE,
    "edit",
    "bearbeiten");       
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Emptying?",
    "Emptying?");
   SetInterString(SpracheDE,
    "Emptying?",
    "Leeren?");   
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Emptying impossible!",
    "Emptying impossible!");
   SetInterString(SpracheDE,
    "Emptying impossible!",
    "Leeren geht nicht!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Either due to problems with filesystem or triggered by user cancellation.",
    "Either due to problems with filesystem or triggered by user cancellation.");
   SetInterString(SpracheDE,
    "Either due to problems with filesystem or triggered by user cancellation.",
    "Entweder durch Probleme des Filesystems oder durch Benutzerabbruch.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "filter",
    "filter");
   SetInterString(SpracheDE,
    "filter",
    "filtern");                    
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "filtered",
    "filtered");
   SetInterString(SpracheDE,
    "filtered",
    "gefiltert"); 
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Filtering files by ratings...",
    "Filtering files by ratings...");
   SetInterString(SpracheDE,
    "Filtering files by ratings...",
    "Files nach Bewertung filtern...");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "inform",
    "inform");
   SetInterString(SpracheDE,
    "inform",
    "informieren");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Insert collection identifier name",
    "Insert collection identifier name");
   SetInterString(SpracheDE,
    "Insert collection identifier name",
    "Sammlungsname eingeben");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "instruct",
    "instruct");
   SetInterString(SpracheDE,
    "instruct",
    "einweisen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "load",
    "load");
   SetInterString(SpracheDE,
    "load",
    "laden");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Loading and extracting files...",
    "Loading and extracting files...");
   SetInterString(SpracheDE,
    "Loading and extracting files...",
    "Files laden und extrahieren...");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Name",
    "Name");
   SetInterString(SpracheDE,
    "Name",
    "Name");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "name",
    "name");
   SetInterString(SpracheDE,
    "name",
    "benennen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "new",
    "new");
   SetInterString(SpracheDE,
    "new",
    "neu");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Pattern of automatically generated filenames: identifier12345.jpg",
    "Pattern of automatically generated filenames: identifier12345.jpg");
   SetInterString(SpracheDE,
    "Pattern of automatically generated filenames: identifier12345.jpg",
    "Muster der generierten Filenamen: Sammlungsname12345.jpg");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Pick JPEGs only from real directory entries.",
    "Pick JPEGs only from real directory entries.");
   SetInterString(SpracheDE,
    "Pick JPEGs only from real directory entries.",
    "Bilder nicht aus dem 'Zuletzt verwendet'-Angebot picken.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Picture",
    "Picture");
   SetInterString(SpracheDE,
    "Picture",
    "Bild");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Picture viewer reports a problem!",
    "Picture viewer reports a problem!");
   SetInterString(SpracheDE,
    "Picture viewer reports a problem!",
    "Bildbetrachter meldet einen Fehler!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Pictures have been inserted into column.\nSave column first.",
    "Pictures have been inserted into column.\nSave column first.");
   SetInterString(SpracheDE,
    "Pictures have been inserted into column.\nSave column first.",
    "Bilder wurden in die Spalte einsortiert.\nSpalte unbedingt zuerst abspeichern.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Program allows to sort large JPEG collections by drag n' drop operations.\nOnly basic and sustainable storage mechanism are used,\ne.g. directories, filenames sorted in alphabetic order, basic Exif-tags.",
    "Program allows to sort large JPEG collections by drag n' drop operations.\nOnly basic and sustainable storage mechanism are used,\ne.g. directories, filenames sorted in alphabetic order, basic Exif-tags.");
   SetInterString(SpracheDE,
    "Program allows to sort large JPEG collections by drag n' drop operations.\nOnly basic and sustainable storage mechanism are used,\ne.g. directories, filenames sorted in alphabetic order, basic Exif-tags.",
    "Programm zur Drag-Drop-Sortierung grosser JPEG-Bildersammlungen.\nBenutzt werden nur die basalen, dauerhaften Speichermechanismen\nwie Verzeichnisse, alphabetisch sortierte Filenamen, Exif-Daten.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Ratings from ... to (inclusive)",
    "Ratings from ... to (inclusive)");
   SetInterString(SpracheDE,
    "Ratings from ... to (inclusive)",
    "Bewertungen von ... bis (einschliesslich)");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Really wanna save?",
    "Really wanna save?");
   SetInterString(SpracheDE,
    "Really wanna save?",
    "Wirklich Speichern?");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Reload?",
    "Reload?");
   SetInterString(SpracheDE,
    "Reload?",
    "Neuladen?");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Reloading impossible!",
    "Reloading impossible!");
   SetInterString(SpracheDE,
    "Reloading impossible!",
    "Neuladen geht nicht!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Renaming files...",
    "Renaming files...");
   SetInterString(SpracheDE,
    "Renaming files...",
    "Files umbenennen...");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Root causes are file- or pixelbuffer-operations.",
    "Root causes are file- or pixelbuffer-operations.");
   SetInterString(SpracheDE,
    "Root causes are file- or pixelbuffer-operations.",
    "Ursachen sind File- oder Pixelpuffer-Operationen.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "rotate ccw",
    "rotate ccw");
   SetInterString(SpracheDE,
    "rotate ccw",
    "links drehen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "rotate cw",
    "rotate cw");
   SetInterString(SpracheDE,
    "rotate cw",
    "rechts drehen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "save",
    "save");
   SetInterString(SpracheDE,
    "save",
    "speichern");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Saving failed!",
    "Saving failed!");
   SetInterString(SpracheDE,
    "Saving failed!",
    "Speichern fehlgeschlagen!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Saving files...",
    "Saving files...");
   SetInterString(SpracheDE,
    "Saving files...",
    "Files speichern...");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "show",
    "show");
   SetInterString(SpracheDE,
    "show",
    "zeigen");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "sort",
    "sort");
   SetInterString(SpracheDE,
    "sort",
    "sortieren");                  
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Sorting files by name / date...",
    "Sorting files by name / date...");
   SetInterString(SpracheDE,
    "Sorting files by name / date...",
    "Files nach Namen / Datum sortieren...");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "System call failed!",
    "System call failed!");
   SetInterString(SpracheDE,
    "System call failed!",
    "Systemaufruf fehlgeschlagen!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "System call reports a problem.",
    "System call reports a problem.");
   SetInterString(SpracheDE,
    "System call reports a problem.",
    "Systemaufruf meldet einen Fehler.");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "Terminate without saving?",
    "Terminate without saving?");
   SetInterString(SpracheDE,
    "Terminate without saving?",
    "Ohne Speichern beenden?");
   //*********************************************************************************************************                  
   SetInterString(SpracheEN, 
    "There are problems with the filesystem.",
    "There are problems with the filesystem.");
   SetInterString(SpracheDE,
    "There are problems with the filesystem.",
    "Es gibt Probleme mit dem Filesystem.");
   //*********************************************************************************************************                  
   SetInterString(SpracheEN, 
    "Unable to create PickSortPro control file!",
    "Unable to create PickSortPro control file!");
   SetInterString(SpracheDE,
    "Unable to create PickSortPro control file!",
    "Anlegen der PickSortPro-Steuerdatei fehlgeschlagen!");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "User",
    "User");
   SetInterString(SpracheDE,
    "User",
    "Benutzer");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "view",
    "view");
   SetInterString(SpracheDE,
    "view",
    "betrachten");
   //*********************************************************************************************************
   SetInterString(SpracheEN, 
    "void",
    "void");
   SetInterString(SpracheDE,
    "void",
    "leer");
   //*********************************************************************************************************
   InterInitialisiert = true;
  }  // if !Initialisiert
}  // GenerellesInternationalisierungClass


GenerellesInternationalisierungClass::~GenerellesInternationalisierungClass()
{
 Phrase* Element;
 // Hier alle Allokationen wieder deallokieren
 for (int Spalte = static_cast<int>(SpracheEN) + 1; Spalte < static_cast<int>(SpracheANZAHL) ; ++Spalte)
  {
   for (register int Zeile = 255; Zeile; --Zeile)
    {
     Element = PhrasenMap[Spalte][Zeile];
     if (Element)
      {
       delete Element;
      }
    }
   Element = PhrasenMap[Spalte][0];
   if (Element)
    {
     delete Element;
    }
  }
}  // ~GenerellesInternationalisierungClass


void GenerellesInternationalisierungClass::SetInterSprache(InterSpracheKuerzel Sprache)
{
 AktuelleSprache = Sprache;
}  // SetSprache


const Glib::ustring& GenerellesInternationalisierungClass::GetInterString(const Glib::ustring& InterStringEN)
{
 if (AktuelleSprache == SpracheANZAHL) 
  {
   return InterStringEN;
  }
 else
  {
   int Zeile = static_cast<int>(TinyFNV1a(InterStringEN));
   Phrase* ElementZeigerNN = PhrasenMap[static_cast<int>(AktuelleSprache)][Zeile];
   Phrase* ElementZeigerEN = PhrasenMap[static_cast<int>(SpracheEN)][Zeile];
   while (ElementZeigerNN)
    {
     if (!ElementZeigerNN->Naechst)
      {
       return ElementZeigerNN->TextNN;
      }
     else
      {     
       if (ElementZeigerEN->TextNN == InterStringEN)
        {
         return ElementZeigerNN->TextNN;
        }
       else
        {
         ElementZeigerNN = ElementZeigerNN->Naechst;
         ElementZeigerEN = ElementZeigerEN->Naechst;
        }
      }
    }
   // Da war ein Fehler bei der Eingabe der Daten in die Hash-Matrix:
   return InterStringEN;   
  }
}  // GetInterString


void GenerellesInternationalisierungClass::SetInterString
(InterSpracheKuerzel Sprache, const Glib::ustring& InterStringEN, const Glib::ustring& InterStringNN)
{
 if (Sprache == SpracheANZAHL)
  {
   return;
  }
 Phrase* Element = new(Phrase);
 if (!Element)
  {
   std::cerr << "GenerellesInternationalisierungsClass: Unable to alloc memory." << std::endl;
   return;
  }
 Element->TextNN = InterStringNN;
 int Zeile = static_cast<int>(TinyFNV1a(InterStringEN));
 Phrase* ElementZeiger = PhrasenMap[static_cast<int>(Sprache)][Zeile];
 // Neues Element vorne in die Kette einhaengen;
 PhrasenMap[static_cast<int>(Sprache)][Zeile] = Element;
 Element->Naechst = ElementZeiger;
}  // SetInterString


// Hash-Berechnung; schaue nach im Internet bei chongo <Landon Curt Noll> /\../\:
uint8_t GenerellesInternationalisierungClass::TinyFNV1a(const Glib::ustring& EnglischerText)
{
 Glib::ustring::const_iterator ETextIterator = EnglischerText.begin();
 uint32_t HashWert = FNV1aStartwert;
 // Fuer die Hashberechnung braucht man nicht die ganze Textzeile als Berechnungsgrundlage.
 // Z.B. 25 Buchstaben tun es auch:
 int Zaehler = 50;
 while ((ETextIterator != EnglischerText.end()) && (Zaehler))
  {
   HashWert ^= static_cast<uint32_t>(*ETextIterator);
   HashWert *= FNV1aPrimzahl;
   ++ETextIterator;
   --Zaehler;
  }
 HashWert = ((HashWert >> 8) ^ HashWert) & ((static_cast<u_int32_t>(1)<<8)-1);
 return static_cast<uint8_t>(HashWert);
}  // TinyFNV1a


// Zugriff auf die Internationalisierung ueber eine globale Variable, die gleich zum Programmstart angelegt
// wird. So werden alle Phrasen gleich zum Programmstart verfuegbar:
GenerellesInternationalisierungClass InterPhrasen;


/*
 * PickSortGenerelles.cpp ************************************************************************************
 */

