/*
 *************************************************************************************************************
 * PickSortGenerelles.h       
 *
 * Allgemeine Vereinbarungen, Konstanten, Grafiken, usf.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#ifndef PICKSORTGENERELLES_H
#define PICKSORTGENERELLES_H

// Erstellt durch Autoheader-Werkzeug:
#include "config.h"

#include <gtkmm.h>
#include <iostream>
#include <stdlib.h>
#include <stdint.h>


class GenerellesClass  
{
 // Nur als Ist-Ein zu erreichen:
 protected:
  GenerellesClass();
  virtual ~GenerellesClass();
  // Farben fuer die Elemente des Benutzerinfaces:
  const static Gdk::RGBA GenerellesSpalteFarbeNormal;
  const static Gdk::RGBA GenerellesSpalteFarbeAktiv;
  const static Gdk::RGBA GenerellesSpalteFarbeGewaehlt;
  const static Gdk::RGBA GenerellesSpalteSchrift;
  const static Gdk::RGBA GenerellesElementFarbeNormal;
  const static Gdk::RGBA GenerellesElementFarbeAktiv;   
  const static Gdk::RGBA GenerellesElementFarbeGewaehlt;
  const static Gdk::RGBA GenerellesElementSchrift;
  // Briefmarke fuer generische Bildchen des Programmes in Schwarz-Weiss und Farbe.
  // Bildchen werden dauerhaft und nur je einmal gehalten:
  static Glib::RefPtr<Gdk::Pixbuf> GenerellesPixelbufferSW;
  static Glib::RefPtr<Gdk::Pixbuf> GenerellesPixelbufferRB;
  static std::vector<Glib::RefPtr<Gdk::Pixbuf> > GenerellesPixelbuffersIcons;
  // Die Spalten müssen wissen, wenn z.B. ein Element per Drag und Drop zu ihnen geschoben wurde.
  // Aber das Wissen generieren die Elemente, die als Variablen in den Spalten instanziert werden,
  // so dass sie nicht direkt auf die Spalten zugreifen koennen. Um eine ueber Spalten und Elemente
  // greifende Informationsbasis zu schaffen, werden hier verschiedene std-Listen definiert,
  // auf die alle Elemente und Spalten gleichermassen zugreifen koennen:
  static std::vector<bool> GenerellesEditierspalteHingeschoben;
  //// static std::vector<bool> GenerellesEditierspalteWeggeschoben;
  static std::vector<bool> GenerellesEditierspalteUmsortiert;
 private:
  // Kontrollvariable, dass die Bildchenfiles nur einmal geladen werden:
  static bool GenerellesInitialisiert;
};  // class PickSortGenerellesClass


//************************************************************************************************************


#define _(Kram) InterPhrasen.GetInterString(Kram)


class GenerellesInternationalisierungClass
{
 public:
  GenerellesInternationalisierungClass();
  virtual ~GenerellesInternationalisierungClass();
  enum InterSpracheKuerzel {SpracheEN = 0, SpracheDE, SpracheANZAHL};
  void SetInterSprache(InterSpracheKuerzel Sprache);  
  const Glib::ustring& GetInterString(const Glib::ustring& InterStringEN);
 private:
  struct Phrase
   {
    Glib::ustring TextNN;
    Phrase* Naechst;
   };
  static Phrase* PhrasenMap[SpracheANZAHL][256];
  static InterSpracheKuerzel AktuelleSprache;
  static bool InterInitialisiert;
  const static uint32_t FNV1aStartwert;
  const static uint32_t FNV1aPrimzahl;
  void SetInterString
   (InterSpracheKuerzel Sprache, const Glib::ustring& InterStringEN, const Glib::ustring& InterStringNN);
  uint8_t TinyFNV1a(const Glib::ustring& EnglischerText);
}; // GenerellesInternationalisierungClass


extern GenerellesInternationalisierungClass InterPhrasen;


#endif  // PICKSORTGENERELLES_H


/*
 * PickSortGenerelles.h **************************************************************************************
 */

