/*
 *************************************************************************************************************
 * PickSortProFilehandling.cpp       
 *
 * Fileinteraktionen fuer das Projektorprogramm plus Grafikaufbereitung in Nebenzeit.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortProFilehandling.h"


PickSortProFilehandlingClass::PickSortProFilehandlingClass
 (const std::string& DatenfileName, int BildschirmBreite, int BildschirmHoehe)
{
 BreiteBildschirm = BildschirmBreite;
 HoeheBildschirm = BildschirmHoehe;
 // Datenfile oeffnen 
 std::ifstream BilderFilenamenDatei(DatenfileName.c_str());
 if (!BilderFilenamenDatei)
  {
   std::cerr << "PickSortProFilehandlingClass::PickSortProFilehandlingClass: Unable to open data file. => exit(1)" << std::endl;
   exit(1);
  }
 // Datenfile auslesen:
 char Zeichenpuffer[256]; 
 while (BilderFilenamenDatei.good())
  {
   BilderFilenamenDatei.getline(Zeichenpuffer, 256);
   // Besondes einfach geht es in einem String zu suchen:
   std::string StringPuffer(Zeichenpuffer);
   // Test darauf, ob eine Verzeichnisangabe in der Zeile vorliegt, alle anderen Zeilen werden ueberlesen.
   // Z.B. Steuerzeichen am Anfang oder Ende des Files:
   if (StringPuffer.find('/') != std::string::npos)
    {
     // Wenn gut, dann lege Filename mit Verzeichnispfad in einem Stringvektor ab:
     FileNamensVektor.push_back(StringPuffer);
    }
  }
 // Nach dem Auslesen das Datenfile schliessen und danach aus dem Verzeichnis loeschen:
 BilderFilenamenDatei.close();
 std::string StringMerker("rm " + DatenfileName);
 int Fehler = system(StringMerker.c_str());
 if (Fehler)
  {
   std::cerr << "PickSortProFilehandlingClass::PickSortProFilehandlingClass: Unable to delete data file." << std::endl;
  }
}  // PickSortProFilehandlingClass


PickSortProFilehandlingClass::~PickSortProFilehandlingClass()
{
}  // ~PickSortProFilehandlingClass 


unsigned int PickSortProFilehandlingClass::GetAnzahlBilder()
{
 return static_cast<unsigned int>(FileNamensVektor.size());
}  // GetAnzahlBilder


Magick::Blob PickSortProFilehandlingClass::GetErstesBild()
{
 if (GetAnzahlBilder() > 0)
  {
   FileNamensIterator = FileNamensVektor.begin();
   AktuellerBlob = GetBild(*FileNamensIterator, BreiteBildschirm, HoeheBildschirm);
   if ((FileNamensIterator + 1) != FileNamensVektor.end())
    {
     FileNamensVeraenderung = +1;
    }
   else
    {
     FileNamensVeraenderung = 0;
    }
   return AktuellerBlob;
  }
 else
  {
   std::cerr << "PickSortProFilehandlingClass::GetErstesBild: File doesn't contain picture data. => exit(1) " << std::endl;
   exit(1);
  }   
}  // GetAktuellesBild


Magick::Blob PickSortProFilehandlingClass::GetNaechstesBild()
{
 if ((FileNamensIterator + 1) != FileNamensVektor.end())
  {
   ++FileNamensIterator;
   VorherigerBlob = AktuellerBlob;
   AktuellerBlob = NaechsterBlob;
   if ((FileNamensIterator + 1) != FileNamensVektor.end())
    {
     FileNamensVeraenderung = +1;
    }
   else
    {
     FileNamensVeraenderung = 0;
    }
  }
 else
  {
   FileNamensVeraenderung = 0;
  }
 return AktuellerBlob; 
}  // GetNaechstesBild


Magick::Blob PickSortProFilehandlingClass::GetVorherigesBild()
{
 if (FileNamensIterator != FileNamensVektor.begin())
  {
   --FileNamensIterator;
   NaechsterBlob = AktuellerBlob;
   AktuellerBlob = VorherigerBlob;
   if (FileNamensIterator != FileNamensVektor.begin())
    {
     FileNamensVeraenderung = -1;
    }
   else
    {
     FileNamensVeraenderung = 0;    
    }
  }
 else
  {
   FileNamensVeraenderung = 0;
  }
 return AktuellerBlob; 
}  // GetVorherigesBild


void PickSortProFilehandlingClass::BilderNachladen()
{
 switch (FileNamensVeraenderung)
  {
   case +1:
    NaechsterBlob = GetBild(*(FileNamensIterator + 1), BreiteBildschirm, HoeheBildschirm);
    return;
   case -1:
    VorherigerBlob = GetBild(*(FileNamensIterator - 1), BreiteBildschirm, HoeheBildschirm);
    return;
  }
}  // BilderNachladen


Magick::Blob PickSortProFilehandlingClass::GetBild
 (std::string& Bildfile, int BildschirmBreite, int BildschirmHoehe)
{
 try
  {
   // Dimensionen des Bilschirmes laden:
   Magick::Geometry BildMasse(BildschirmBreite, BildschirmHoehe);
   // Anweisung 'Wenn Bild groesser, dann entsprechend beschneiden' laden:
   BildMasse.greater();
   // Bild von der Festplatte laden:
   Magick::Image Bild(Bildfile); 
   // Wenn Bild zu groß, dann nach dem Bildschirm skalieren:
   Bild.resize(BildMasse);
   // Unterschiedliche Koordinatenurspruenge von Magick++ und OpenGL (in Nebenzeit) anpassen:
   Bild.flip();
   // Bild muss gleich noch ins Zentrum des Bildschirms gerueckt werden:
   int VersatzHorizontal = (BildschirmBreite - static_cast<int>(Bild.columns())) / 2;
   int VersatzVertikal = (BildschirmHoehe - static_cast<int>(Bild.rows())) / 2;                     
   // Schwarze Balken einfuegen. (Anmerkung: Die Gravity-Parameter funktionieren nicht):                        
   Bild.extent(BildMasse, "black");
   // Das Bild nun noch ins Zentrum des Bildschirms rollen:                           
   Bild.roll(VersatzHorizontal, VersatzVertikal);
   // OpenGL-vertraegliches Grafik-Format waehlen:
   Bild.magick("RGBA");
   // Ein leeres Blob-Objekt schaffen, das die Daten dann an OpenGL uebergeben wird:
   Magick::Blob RGBBlob;  
   // Bild in den Blob schreiben:
   Bild.write(&RGBBlob);
   return RGBBlob;   
  }
 catch (...)
  {
   std::cerr << "PickSortProFilehandlingClass::GetBild: Unable to open / read picture file => exit(1)." << std::endl;
   exit(1);
  }
}  // GetBild


/*
 * PickSortProFilehandling.cpp *******************************************************************************
 */

