/*
 *************************************************************************************************************
 * PickSortProMain.cpp
 * Copyright (C) 2014 Andreas Eichendorf <PickSort@Andreas-Petra.de>
 * 
 * PickSort is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PickSort is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *************************************************************************************************************
 */


// Erstellt durch Autoheader-Werkzeug:
#include "config.h"

#include <iostream>
#include <Magick++.h>

#include "PickSortProFilehandling.h"
#include "PickSortProXOpenGL.h"
#include "PickSortProTimer.h"


int main(int argc, char* argv[])
{
 // Ueber die Kommandozeile den Weg zu einem Textfile erwarten, in dem die Filenamen mit absoluten
 // Pfaden aufgefuehrt sind. Pro File steht eine Zeile mit abschliessendem Zeilenumbruch im Textfile:
 if (argc < 2)
  {
   std::cerr << "PickSortProMain::main: Program call without control data file name. => exit(1)" << std::endl;
   exit(1);
  }
 // Den JPEG-Lader initialisieren:
 Magick::InitializeMagick(argv[0]);
 // Das X11-Fenster oeffnen, die OpenGL-Umgebung schaffen, Zykluszeit der Grafik des Rechners messen: 
 PickSortProXOpenClass ProjektionsFenster;
 // Das File mit den Bilderadressen oeffnen, die Bilderadressen laden und am Ende das File loeschen:
 PickSortProFilehandlingClass BilderDaten(argv[1], ProjektionsFenster.GetBildschirmBreite(),
                                                   ProjektionsFenster.GetBildschirmHoehe());
 // Handler fuer die Tastenbefehle des Benutzers definieren:
 XBEFEHLE BenutzerWunsch;
 // Timerfunktionen initialisieren:
 PickSortProTimerClass Timer;
 // Anfangszustand der Bilder herstellen und erstes Bild anzeigen:
 Magick::Blob NeuesBild = BilderDaten.GetErstesBild();
 ProjektionsFenster.XBildschirmUeberblenden(NeuesBild);
 BilderDaten.BilderNachladen();
 Timer.SetHauptZyklusStart();
 for(;;)
  {
   // X-Tastenspeicher pruefen und verarbeiten:
   if (ProjektionsFenster.XTastaturAbfrage(&BenutzerWunsch))
    {
     switch (BenutzerWunsch)
      {
       case NAECHSTESBILD:        
        NeuesBild = BilderDaten.GetNaechstesBild();
        ProjektionsFenster.XBildschirmUeberblenden(NeuesBild);        
        BilderDaten.BilderNachladen();
        Timer.SetHauptZyklusStart();
        break;
       case VORHERIGESBILD:
        NeuesBild = BilderDaten.GetVorherigesBild();
        ProjektionsFenster.XBildschirmUeberblenden(NeuesBild);
        BilderDaten.BilderNachladen();
        Timer.SetHauptZyklusStart();
        break;
       case ERSTESBILD:
        NeuesBild = BilderDaten.GetErstesBild();
        ProjektionsFenster.XBildschirmUeberblenden(NeuesBild);
        BilderDaten.BilderNachladen();
        Timer.SetHauptZyklusStart();
        break;
       case LAENGEREDAUER:
        Timer.SetHauptZyklusLaenger();
        break;
       case KUERZEREDAUER:
        Timer.SetHauptZyklusKuerzer();
        break;
       case PAUSE:
        Timer.SetHauptZyklusStop();
        break;
       case WEITER:
        Timer.SetHauptZyklusWeiter();
        Timer.SetHauptZyklusEnde();
        break;
       case ENDE:       
        exit(0);
      }
    }
   // Wenn es Zeit ist, auf das nächste Bild überblenden:
   if (Timer.GetHauptZyklusZuSchalten())
    {
     // In Nebenzeit vorbereitetes Bild holen:
     NeuesBild = BilderDaten.GetNaechstesBild();
     ProjektionsFenster.XBildschirmUeberblenden(NeuesBild);
     // Moegliche naechste Bilder in Nebenzeit vorbereiten:
     BilderDaten.BilderNachladen();
    }
   // Zykluszeit einstellen, dazwischen die Ressourcen freigeben:
   Timer.HauptZyklusVerzoegern();
  }
}  // main


/*
 * PickSortProMain.cpp ***************************************************************************************
 */

