/*
 *************************************************************************************************************
 * PickSortProSmartRAM.cpp      
 *
 * Eine Smart-Pointer-Klasse fuer allgemeine RAM-Verwaltung. Speichereinheit ist unsigned char.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortProSmartRAM.h"


PickSortProSmartRAMClass::PickSortProSmartRAMClass()
{
 Zeilen = 0;
 Spalten = 0;
 Farben = 0;
 Elemente = 0;
 SmartRAM = 0;
 AnzahlReferenzen = new unsigned long;
 *AnzahlReferenzen = 1;
}  // PickSortProSmartRAMClass


PickSortProSmartRAMClass::PickSortProSmartRAMClass(unsigned long AnzahlElemente, unsigned long AnzahlFarben)
{
 try
  {
   SmartRAM = new unsigned char[AnzahlElemente];
  }
 catch (...)
  {
   std::cerr << "PickSortProSmartRAMClass::PickSortProSmartRAMClass: RAM allocation failed. => exit(1)" << std::endl;
   exit(1);
  }
 Zeilen = new unsigned long;
 *Zeilen = 1;
 Spalten = new unsigned long;
 *Spalten = AnzahlElemente / AnzahlFarben;
 Farben = new unsigned long;
 *Farben = AnzahlFarben;
 Elemente = new unsigned long;
 *Elemente = AnzahlElemente;
 AnzahlReferenzen = new unsigned long;
 *AnzahlReferenzen = 1; 

}  // PickSortProSmartRAMClass


PickSortProSmartRAMClass::PickSortProSmartRAMClass
 (unsigned long AnzahlZeilen, unsigned long AnzahlSpalten, unsigned long AnzahlFarben)
{
 if (AnzahlZeilen < 1)
  {
   AnzahlZeilen = 1;
  }
 if (AnzahlSpalten < 1)
  {
   AnzahlSpalten = 1;
  }
 if (AnzahlFarben < 1)
  {
   AnzahlFarben = 1;
  }
 if (AnzahlFarben > 4)
  {
   AnzahlFarben = 4;
  }
 Elemente = new unsigned long;
 *Elemente = AnzahlZeilen * AnzahlSpalten * AnzahlFarben;
 try
  {
   SmartRAM = new unsigned char[*Elemente];
  }
 catch (...)
  {
   std::cerr << "PickSortProSmartRAMClass::PickSortProSmartRAMClass: RAM allocation failed. => exit(1)" << std::endl;
   exit(1);
  }
 Zeilen = new unsigned long;
 *Zeilen = AnzahlZeilen;
 Spalten = new unsigned long;
 *Spalten = AnzahlSpalten;
 Farben = new unsigned long;
 *Farben = AnzahlFarben;
 AnzahlReferenzen = new unsigned long;
 *AnzahlReferenzen = 1;
}  // PickSortProSmartRAMClass


PickSortProSmartRAMClass::PickSortProSmartRAMClass(const PickSortProSmartRAMClass &Objekt)
{
 SmartRAM = Objekt.SmartRAM;
 Zeilen = Objekt.Zeilen;
 Spalten = Objekt.Spalten;
 Farben = Objekt.Farben;
 Elemente = Objekt.Elemente;
 AnzahlReferenzen = Objekt.AnzahlReferenzen;
 ++(*AnzahlReferenzen);
}  // PickSortProSmartRAMClass


PickSortProSmartRAMClass::~PickSortProSmartRAMClass()
{
 --(*AnzahlReferenzen);
 if (!(*AnzahlReferenzen))
  {
   delete[] SmartRAM;
   delete Zeilen;
   delete Spalten;
   delete Farben;
   delete Elemente;
   delete AnzahlReferenzen;
  }
}  // ~PickSortProSmartRAMClass


PickSortProSmartRAMClass& PickSortProSmartRAMClass::operator=(const PickSortProSmartRAMClass &Objekt)
{
 // Hier zuerst auf Gleichheit der Adressen testen, so dass der Zaehler nicht zu hoch gezaehlt wird:
 if (this != &Objekt)
  {
   --(*AnzahlReferenzen);
   if (!(*AnzahlReferenzen))
    {
     delete[] SmartRAM;
     delete Zeilen;
     delete Spalten;
     delete Farben;
     delete Elemente;
     delete AnzahlReferenzen;
    }
   SmartRAM = Objekt.SmartRAM;
   Zeilen = Objekt.Zeilen;
   Spalten = Objekt.Spalten;
   Farben = Objekt.Farben;
   Elemente = Objekt.Elemente;
   AnzahlReferenzen = Objekt.AnzahlReferenzen;
   ++(*AnzahlReferenzen);
   return *this;
  }
 else 
  {
   return *this;
  }
}  // operator=


unsigned long PickSortProSmartRAMClass::GetAnzahlZeilen()
{
 return *Zeilen;
}  // GetAnzahlZeilen


unsigned long PickSortProSmartRAMClass::GetAnzahlSpalten()
{
 return *Spalten;
}  // GetAnzahlSpalten


unsigned long PickSortProSmartRAMClass::GetAnzahlFarben()
{
 return *Farben;
}  // GetAnzahlFarben


unsigned long PickSortProSmartRAMClass::GetAnzahlElemente()
{
 return *Elemente;
}  // GetAnzahlElemente


unsigned long PickSortProSmartRAMClass::GetAnzahlReferenzen()
{
 return *AnzahlReferenzen;
}  // GetAnzahlInstanzen


unsigned char* PickSortProSmartRAMClass::GetAnfang()
{
 return SmartRAM;
}  // GetAnfang


unsigned char* PickSortProSmartRAMClass::GetZeilenAnfang(unsigned long Zeile)
{
 if (Zeile >= *Zeilen)
  {
   Zeile = *Zeilen - 1;
  }
 return &(SmartRAM[Zeile * (*Spalten) * (*Farben)]);
}  // ZeilenAnfang


bool PickSortProSmartRAMClass::NichtNULL()
{
 if (SmartRAM)
  {
   return true;
  }
 else
  {
   return false;
  }
}  // NichtNULL

/*
 * PickSortProSmartRAM.cpp *********************************************************************************
 */

