/*
 *************************************************************************************************************
 * PickSortProSmartRAM.h       
 *
 * Eine Smart-Pointer-Klasse fuer die allgemeine RAM-Verwaltung. Speichereinheit ist unsigned char.
 * 
 * Ueberlegungen:
 * Man darf Smart-Pointer mit internem Instanzenzaehler keinen Nicht-initialisierten Zeigern zuweisen,
 * denn so kaeme die Zaehlung voellig durcheinander. Am besten also nur Variablen auf dem Stack haben und
 * Referenzen darauf verwenden.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#ifndef PICKSORPROSMARTRAM_H
#define PICKSORPROSMARTRAM_H

#include <stdlib.h>
#include <iostream>


class PickSortProSmartRAMClass
{
 public:
  // Konstruktor fuer eine Instanz ohne Allokation von RAM, nur die Kontrollvariablen werden aktiviert:
  PickSortProSmartRAMClass();
  // Konstruktoren, die RAM allokieren:
  // AnzahlElemente sind Zeilen mal Spalten mal Farben:
  PickSortProSmartRAMClass(unsigned long AnzahlElemente, unsigned long AnzahlFarben = 1);  
  PickSortProSmartRAMClass
   (unsigned long AnzahlZeilen, unsigned long AnzahlSpalten, unsigned long AnzahlFarben);
  // Copy-Konstruktor:
  PickSortProSmartRAMClass(const PickSortProSmartRAMClass &Objekt);
  virtual ~PickSortProSmartRAMClass();
  // Zuweisung:
  PickSortProSmartRAMClass& operator=(const PickSortProSmartRAMClass& Objekt);
  // Getter:
  unsigned long GetAnzahlZeilen();
  unsigned long GetAnzahlSpalten();
  unsigned long GetAnzahlFarben();
  unsigned long GetAnzahlElemente();
  unsigned long GetAnzahlReferenzen();
  // Gibt Zeiger auf den Anfang des Arrays heraus:
  unsigned char* GetAnfang();
  // Gibt Zeiger auf den gewuenschten Zeilenvektor heraus. Zeilenzaehlung startet bei 0:
  unsigned char* GetZeilenAnfang(unsigned long Zeile = 0);
  // Test auf NULL; es koennte ja doch einmal jemand einen nicht allokierten Zeiger einrichten wollen:
  bool NichtNULL();
 private:
  unsigned long* Zeilen;
  unsigned long* Spalten;
  unsigned long* Farben;
  unsigned long* Elemente;
  unsigned long* AnzahlReferenzen;
  unsigned char* SmartRAM;  
};  // PickSortProSmartRAMClass


#endif  // PICKSORPROSMARTRAM_H


/*
 * PickSortProSmartRAM.h *************************************************************************************
 */

