/*
 *************************************************************************************************************
 * PickSortProTimer.cpp       
 *
 * Timerfunktionen fuer das Projektorprogramm.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#include "PickSortProTimer.h"

// 1 ist ein Sicherheitswert: Sollte nicht initialisiert worden sein, dann gibt es kein
// Teilen durch 0 aber auch ein merkbar falsches Verhalten des Schleifentimings:
unsigned int PickSortProTimerClass::USekundenProFrame = 1;
// Bei 50 Frames pro Sekunde sieht auch ein noch junges Auge kein Flimmern mehr.
// Daher ist das als Standardwert vorgesehen, umgerechnet in die Verzoegerungszeit, also 20.000µs :
const unsigned int PickSortProTimerClass::MaxUSekundenUeberblendung = 20000;
// Zeit in µ-Sekunden fuer die komplette Ueberblendung von einem zum naechsten Bild:
const unsigned int PickSortProTimerClass::USekundenGesamtUeberblendung = 2000000;
// Anzahl der Schleifen fuer eine komplette Ueberblendung. Wird zentral berechnet, hier Sicherheitswert:
unsigned int PickSortProTimerClass::AnzahlGesamtUeberblendSchleifen = 10;
// Zeit in µ-Sekunden, in der sich eine Hauptschleife dreht. Das ist dann für den Benutzer
// die merkbare Tasten-Totzeit. Achtung: Die Totzeit darf wg. usleep() max. 1 Sekunde betragen:
const unsigned int PickSortProTimerClass::USekundenHauptschleife = 250000;
// 20 Hauptschleifen mit 1/2 Sekunden ergeben 10 Sekunden Repetierzeit je Dia als Startwert: 
const unsigned int PickSortProTimerClass::AnzahlHauptSchleifenVorgabe = 40;



PickSortProTimerClass::PickSortProTimerClass()
{
 ZustandsZaehler = 0;
 HauptZyklusWeiter = true;
 AnzahlHauptSchleifen = AnzahlHauptSchleifenVorgabe;
 HauptSchleifenZaehler = AnzahlHauptSchleifen;
}  // PickSortProTimerClass


PickSortProTimerClass::~PickSortProTimerClass()
{
}  // ~PickSortProTimerClass


void PickSortProTimerClass::ZeitnahmeStartKurz()
{
 ZeitStartKurz = time(NULL); 
 ZustandsZaehler = 1;
}


void PickSortProTimerClass::ZeitnahmeEndeKurz(unsigned int FramesKurzZeit)
{
 ZeitEndeKurz = time(NULL);
 FramesKurz = FramesKurzZeit;
 ZustandsZaehler += 2;
}  // TimerZeitnahmeStartKurz


void PickSortProTimerClass::ZeitnahmeStartLang()
{
 ZeitStartLang = time(NULL);
 ZustandsZaehler += 4;
}  // TimerZeitnahmeStartLang


void PickSortProTimerClass::ZeitnahmeEndeLang(unsigned int FramesLangZeit)
{
 ZeitEndeLang = time(NULL);
 FramesLang = FramesLangZeit;
 ZustandsZaehler += 8;
 if (ZustandsZaehler != 15)
  {
   ZustandsZaehler = 0;
   USekundenProFrame = 0;
   return;   
  }
 double SekundenLang = difftime(ZeitEndeLang, ZeitStartLang);
 if (SekundenLang <= 0.0)
  {
   ZustandsZaehler = 0;
   USekundenProFrame = 0;
   return;  
  }
 double SekundenKurz = difftime(ZeitEndeKurz, ZeitStartKurz);
 if (SekundenKurz <= 0.0)
  {
   ZustandsZaehler = 0;
   USekundenProFrame = 0;
   return;  
  }   
 double Sekunden = SekundenLang - SekundenKurz;
 if (Sekunden <= 0.0)
  {
   ZustandsZaehler = 0;
   USekundenProFrame = 0;
   return;
  }
 if (FramesLang <= FramesKurz)
  {
   ZustandsZaehler = 0;
   USekundenProFrame = 0;
   return;   
  }
 USekundenProFrame =
  static_cast<unsigned int>(floor(Sekunden * 1000000.0 / static_cast<double>(FramesLang - FramesKurz)));
 // Berechnung der Zyklen fuer eine komplette Ueberblendung:
 if (USekundenProFrame > MaxUSekundenUeberblendung)
  {
   AnzahlGesamtUeberblendSchleifen = USekundenGesamtUeberblendung / USekundenProFrame;
  }
 else
  {
   AnzahlGesamtUeberblendSchleifen = USekundenGesamtUeberblendung / MaxUSekundenUeberblendung;
  }
}  // TimerZeitnahmeEndeLang


unsigned int PickSortProTimerClass::GetUSekundenProFrame()
{
 return USekundenProFrame;
}  // GetTimerUSekundenProFrames


unsigned int PickSortProTimerClass::GetUSekundenProUeberblendung()
{
 if (USekundenProFrame > MaxUSekundenUeberblendung)
  {
   return USekundenProFrame;
  }
 else
  {
   return MaxUSekundenUeberblendung;
  }
}  // GetTimerUSekundenProUeberblendung


unsigned int PickSortProTimerClass::GetZyklenGesamtUeberblendung()
{
 return AnzahlGesamtUeberblendSchleifen;
}  // GetZyklenUeberblendung

void PickSortProTimerClass::UeberblendungsZyklusVerzoegern()
{
 if (MaxUSekundenUeberblendung > USekundenProFrame)
  {
   usleep(MaxUSekundenUeberblendung - USekundenProFrame);
  } 
}  // TimerUeberblendungVerzoegern


void PickSortProTimerClass::HauptZyklusVerzoegern()
{
 usleep(USekundenHauptschleife);
 if (HauptZyklusWeiter)
  {
   if (!HauptSchleifenZaehler)
    {
     HauptSchleifenZaehler = AnzahlHauptSchleifen;
    }
   --HauptSchleifenZaehler;
  }
}  // HauptZyklusVerzoegern


void PickSortProTimerClass::SetHauptZyklusStart()
{
 HauptSchleifenZaehler = AnzahlHauptSchleifen;
}  // ResetHauptZyklus


void PickSortProTimerClass::SetHauptZyklusEnde()
{
 HauptSchleifenZaehler = 0;
}  // ResetHauptZyklus


void PickSortProTimerClass::SetHauptZyklusStop()
{
 HauptZyklusWeiter = false;
}  // SetHauptZyklusStop


void PickSortProTimerClass::SetHauptZyklusWeiter()
{
 HauptZyklusWeiter = true;
 HauptSchleifenZaehler = 0;
}  // SetHauptZyklusWeiter


void PickSortProTimerClass::SetHauptZyklusLaenger()
{
 // Immer eine Sekunde laenger:
 AnzahlHauptSchleifen = AnzahlHauptSchleifen + static_cast<unsigned int>(1000000) / USekundenHauptschleife;
}  // SetHauptZyklusLaenger


void PickSortProTimerClass::SetHauptZyklusKuerzer()
{
 unsigned int HauptZyklenProSekunde = static_cast<unsigned int>(1000000) / USekundenHauptschleife;
 if (AnzahlHauptSchleifen > HauptZyklenProSekunde)
  {
   AnzahlHauptSchleifen = AnzahlHauptSchleifen - HauptZyklenProSekunde;
  }
}  // SetHauptZyklusKuerzer


bool PickSortProTimerClass::GetHauptZyklusZuSchalten()
{
 if (!HauptSchleifenZaehler)
  {
   return true;
  }
  else
  {
   return false;
  }
}  // HauptZyklusSchalten


/*
 * PickSortProTimer.cpp***************************************************************************************
 */
