/*
 *************************************************************************************************************
 * PickSortProTimer.h      
 *
 * Timerfunktionen fuer das Projektorprogramm.
 * Ein detaillierterer Bild-Uebergang und ein weniger detaillierter BildUebergang muessen gemessen werden.
 * Die Differenz davon dividiert durch die Frame-Differenz gibt die Netto-Zeit des einzelnen Frames.
 * Eine vom Modul berechnete usleep-Zeit kann anschliessend benutzt werden.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */


#ifndef PICKSORPROTIMER_H
#define PICKSORPROTIMER_H


#include <time.h>
#include <unistd.h>
#include <math.h>


class PickSortProTimerClass
{
 public:
  PickSortProTimerClass();
  virtual ~PickSortProTimerClass();
  // UEBERBLENDEN:
  // Zwei Zeitnehmerzyklen, eine mit wenigen, eine mit vielen Zyklen. Die Differenzzeit und die Zyklen-
  // differnez lassen die Netto-Bildwechselfrequenz berechnen:
  void ZeitnahmeStartKurz();
  void ZeitnahmeEndeKurz(unsigned int FramesKurzZeit);
  void ZeitnahmeStartLang();
  void ZeitnahmeEndeLang(unsigned int FramesLangZeit);
  // Mikrosekunden pro Frame. Wert muss zuerst mit den Zeitnahme...-Routinen ermittelt werden:
  unsigned int GetUSekundenProFrame();
  // Schneller als 50Hz braucht die Bildwechselfrequenz nicht zu sein. Rueckgabewert ist also >= 20 ms 
  unsigned int GetUSekundenProUeberblendung();
  // Anzahlder Zyklen, die gebraucht werden fuer eine Ueberblendung
  unsigned int GetZyklenGesamtUeberblendung();
  // Einbau in die Ueberblendschleife:
  void UeberblendungsZyklusVerzoegern();
  // HAUPTZYKLUS:
  // Einbau in die Hauptschleife:
  void HauptZyklusVerzoegern();
  // Setzte den Hauptzyklenzaehler auf den Anfang:
  void SetHauptZyklusStart();
  // Setze den Hauptzyklenzaehler auf das Ende - sofortiges Bildumschalten folgt:
  void SetHauptZyklusEnde();  
  // Hauptzyklus stoppen:
  void SetHauptZyklusStop();
  // Hauptzyklus fortsetzen:
  void SetHauptZyklusWeiter();
  // Hauptzyklenzahlen vergroessern und verkleinern:
  void SetHauptZyklusLaenger();
  void SetHauptZyklusKuerzer();
  // Wenn true zurueckkommt, dann muss in der Hauptschleife weitergeschaltet werden:
  bool GetHauptZyklusZuSchalten();
 private:
  unsigned short ZustandsZaehler;
  time_t ZeitStartKurz;
  time_t ZeitEndeKurz;
  time_t ZeitStartLang;
  time_t ZeitEndeLang;
  unsigned int FramesKurz;
  unsigned int FramesLang;
  unsigned int HauptSchleifenZaehler;
  bool HauptZyklusWeiter;
  unsigned int AnzahlHauptSchleifen;
  static unsigned int USekundenProFrame;
  static unsigned int AnzahlGesamtUeberblendSchleifen;
  // UEBERBLENDSCHLEIFE:
  // Zeit, in der eine ganze Ueberblendung stattfinden soll:
  static const unsigned int MaxUSekundenUeberblendung;
  // Schneller als 50Hz muss Ueberblendung nicht laufen. Hier leigt der maximalwert von 20ms:
  static const unsigned int USekundenGesamtUeberblendung;
  // HAUPTSCHLEIFE:
  // Verzoegerungszeit in der Hauptschleife:
  static const unsigned int USekundenHauptschleife;
  // Startwert fuer die Hauptschleifenzahl bis zum Weiterschalten:
  static const unsigned int AnzahlHauptSchleifenVorgabe;
};  // TimerClass


#endif  // PICKSORPROTIMER_H

/*
 * PickSortProTimer.h ****************************************************************************************
 */

