/*
 *************************************************************************************************************
 * PickSortProXOpenGL.h       
 *
 * OpenGL- und X11-Funktionen fuer das Projektorprogramm.
 *
 * Copyright (C) Andreas Eichendorf 2014 <PickSort@Andreas-Petra.de>            
 *************************************************************************************************************
 */

#ifndef PICKSORPROXOPENGL_H
#define PICKSORPROXOPENGL_H

#include <stdlib.h>
#include <string.h>
#include <vector>
#include <iostream>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include <GL/glx.h>
#include <GL/glu.h>
#include <X11/Xmu/WinUtil.h>
#include <X11/extensions/Xrandr.h>

#include <Magick++.h>

#include "PickSortProFilehandling.h"
#include "PickSortProSmartRAM.h"
#include "PickSortProTimer.h"


// Tastencodes, die das X-System zurueckgibt, global verfuegbar:
enum XBEFEHLE
  {          
   FALSCHETASTE   = 0x0000,  // Macros aus X11/keysymdef.h:  
   NAECHSTESBILD  = 0xff56,  // XK_Page_Down
   VORHERIGESBILD = 0xff55,  // XK_Page_Up
   ERSTESBILD     = 0xff50,  // XK_Home
   LAENGEREDAUER  = 0xff52,  // XK_Up
   KUERZEREDAUER  = 0xff54,  // XK_Down
   PAUSE          = 0xff51,  // XK_Left
   WEITER         = 0xff53,  // XK_Right
   ENDE           = 0xff1b   // XK_Escape  
  };


class PickSortProXOpenClass
{
 public:
  PickSortProXOpenClass();
  virtual ~PickSortProXOpenClass();
  bool XTastaturAbfrage(XBEFEHLE* BenutzerWunsch);
  int GetBildschirmBreite();
  int GetBildschirmHoehe();
  void XBildschirmUeberblenden(Magick::Blob NeuesBild);  
 protected:
  Display* XSchnittstelle; 
  Window WurzelFenster;
  Window Fenster;
  std::vector<GLint*> Attribute;
  XVisualInfo* DarstellungsInfo;
  Colormap Farbtafel; 
  XSetWindowAttributes XAttributeSet;  
  GLXContext GLKontext;
  XWindowAttributes WurzelfensterAttribute;
  XEvent XSystemEvent;
  int BildschirmBreite;
  int BildschirmHoehe;
  double BildschirmSeitenverhaeltnis;
  Pixmap QuelleMaske;
 private:
  static bool Initialisiert; 
}; // PickSortProXOpenClass  


#endif  // PICKSORPROXOPENGL_H


/*
 * PickSortProXOpenGL.h **************************************************************************************
 */
